#!/usr/bin/env python

#-------------------------------------------------------#
#                                                       #
#              Name: Pi_Scratch.py                      #
#         Author: James Clarke, Pridopia.               #
#       Website: http://www.pridopia.co.uk              #
#              Date: 10 / 02 / 15                       #
#                Version: 2.74                          #
#                                                       #
#-------------------------------------------------------#
#
#    Patch Notes 2.74:
#
#    Fixed GPIO 7, 8, 9, 10, and 11 I/O Control
#    Supported Raspberry Pi model A+ ( Adafruit Library )
#
# Copyright 2013 Pridopia (www.pridopia.co.uk)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# /LICENSE.txt
#
# http://www.pridopia.com/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIC,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either-express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from __future__ import division
import os
import re
import sys
import time
import glob
import smbus
import spidev
import string
import signal
import socket
import lib.I2C
import binascii
from Tkinter import Tk
from array import array
import RPi.GPIO as GPIO
from subprocess import PIPE
from subprocess import Popen
from subprocess import call
from lib.MCP3002 import MCP3002
from lib.Ada_BMP85 import BMP85
from lib.Ada_PWM_Driver import PWM
from lib.Ada_ADS1x15 import Ads1x15
from subprocess import call as Call
from webiopi.devices.analog import MCP4725
from webiopi.devices.analog import MCP4922
from lib.Adafruit_CharLCD import Adafruit_CharLCD
from lib.Adafruit_CharLCD_V2 import Adafruit_CharLCD_V2
from lib.Adafruit_CharLCDPlate import Adafruit_CharLCDPlate

root = Tk()
root.withdraw()

Bus = smbus.SMBus(1)
Connected = False
Setup = False
PORT = 42001
HOST = "localhost"
Sent = 0
Finished = 0
Failed = 0
SMin = 200
SCen = 325
SMax = 600
PIRList = []
LedPWM1 = {}
TraceList = []
SAngles = [SCen]*32
SAngles2 = [SCen]*32
LIRCPid = ''
SLedrssPID = False
LedrssPID = False
TimePID = 9999
STimePID = 9999
ScrollPID = 9999
GLedPwmPID = 9999
LedMessagePID = 9999
SLedMessagePID = 9999
MPU6050PID = 0
RFIDPid = 0
DHTPin = False
pwm = PWM(0x41, debug=False)
pwm2 = PWM(0x40, debug=False)
pwm.setPWMFreq(60)
pwm2.setPWMFreq(60)
AdcRead = bytearray()
GPIO.setwarnings(False)
GPIO_ECHO = 0
GPIO_TRIGGER = 0
Bit8Setup = False
ProcRunning = False
Lcd2004 = False
RSSRunning = False
RSSRunningA = False
RSSRunningB = False
RSSRunningC = False
RSSRunningD = False
ScrollOn = False
GLedPwmRunning = False

AdcRead.append(0x00)
AdcRead.append(0x00)
AdcRead.append(0x00)
AdcRead.append(0x00)
spi = spidev.SpiDev()
spi.open(0, 1)

Path = os.path.dirname(os.path.abspath(__file__))
if os.getcwd() == Path:
    print "Paths match, Continue."
else:
    os.chdir(Path)
    print "Paths do not match, Changing Directory."

Addons = {
    "RTC": False,
    "0x20": False,
    "0x21": False,
    "0x22": False,
    "Servo": False,
    "DHT22": False,
    "TMP102": False,
    "BMP085": False,
    "Led320": False,
    "Led32S": False,
    "TRACEIR": False,
    "MCP4725": False,
    "MCP3002": False,
    "MCP4802": False,
    "MPU6050": False,
    "ADS1015": False,
    "lcd8544": False,
    "lcd1602": False,
    "Lcd2004": False,
    "Lcd2004A": False,
    "Lcd2004B": False,
    "Lcd2004C": False,
    "Lcd2004D": False,
    "lcd1602g": False,
    "lcd2004g": False,
    "8Switch0": False,
    "8SwitchS": False,
    "MCP3424-1": False,
    "MCP3424-2": False,
    "SpiButton": False,
    "I2CButton": False,
    "PIRSensor": False,
    "LineSensor": False,
    "UltraSonic": False,
    "16x16Matrix": False,
    "24x16Matrix": False
}

font = [
    0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x5f, 0x00, 0x00,  # !
    0x00, 0x07, 0x00, 0x07, 0x00,  # "
    0x14, 0x7f, 0x14, 0x7f, 0x14,  # #
    0x24, 0x2a, 0x7f, 0x2a, 0x12,  # $
    0x23, 0x13, 0x08, 0x64, 0x62,  # %
    0x36, 0x49, 0x55, 0x22, 0x50,  # &
    0x00, 0x05, 0x03, 0x00, 0x00,  # '
    0x00, 0x1c, 0x22, 0x41, 0x00,  # (
    0x00, 0x41, 0x22, 0x1c, 0x00,  # )
    0x14, 0x08, 0x3e, 0x08, 0x14,  # *
    0x08, 0x08, 0x3e, 0x08, 0x08,  # +
    0x00, 0x50, 0x30, 0x00, 0x00,  # ,
    0x08, 0x08, 0x08, 0x08, 0x08,  # -
    0x00, 0x60, 0x60, 0x00, 0x00,  # .
    0x20, 0x10, 0x08, 0x04, 0x02,  # /
    0x3e, 0x51, 0x49, 0x45, 0x3e,  # 0
    0x00, 0x42, 0x7f, 0x40, 0x00,  # 1
    0x42, 0x61, 0x51, 0x49, 0x46,  # 2
    0x21, 0x41, 0x45, 0x4b, 0x31,  # 3
    0x18, 0x14, 0x12, 0x7f, 0x10,  # 4
    0x27, 0x45, 0x45, 0x45, 0x39,  # 5
    0x3c, 0x4a, 0x49, 0x49, 0x30,  # 6
    0x01, 0x71, 0x09, 0x05, 0x03,  # 7
    0x36, 0x49, 0x49, 0x49, 0x36,  # 8
    0x06, 0x49, 0x49, 0x29, 0x1e,  # 9
    0x00, 0x36, 0x36, 0x00, 0x00,  # :
    0x00, 0x56, 0x36, 0x00, 0x00,  # ;
    0x08, 0x14, 0x22, 0x41, 0x00,  # <
    0x14, 0x14, 0x14, 0x14, 0x14,  # =
    0x00, 0x41, 0x22, 0x14, 0x08,  # >
    0x02, 0x01, 0x51, 0x09, 0x06,  # ?
    0x32, 0x49, 0x79, 0x41, 0x3e,  # @
    0x7E, 0x11, 0x11, 0x11, 0x7E,  # A
    0x7F, 0x49, 0x49, 0x49, 0x36,  # B
    0x3E, 0x41, 0x41, 0x41, 0x22,  # C
    0x7F, 0x41, 0x41, 0x22, 0x1C,  # D
    0x7F, 0x49, 0x49, 0x49, 0x41,  # E
    0x7F, 0x09, 0x09, 0x09, 0x01,  # F
    0x3E, 0x41, 0x49, 0x49, 0x7A,  # G
    0x7F, 0x08, 0x08, 0x08, 0x7F,  # H
    0x00, 0x41, 0x7F, 0x41, 0x00,  # I
    0x20, 0x40, 0x41, 0x3F, 0x01,  # J
    0x7F, 0x08, 0x14, 0x22, 0x41,  # K
    0x7F, 0x40, 0x40, 0x40, 0x40,  # L
    0x7F, 0x02, 0x0C, 0x02, 0x7F,  # M
    0x7F, 0x04, 0x08, 0x10, 0x7F,  # N
    0x3E, 0x41, 0x41, 0x41, 0x3E,  # O
    0x7F, 0x09, 0x09, 0x09, 0x06,  # P
    0x3E, 0x41, 0x51, 0x21, 0x5E,  # Q
    0x7F, 0x09, 0x19, 0x29, 0x46,  # R
    0x46, 0x49, 0x49, 0x49, 0x31,  # S
    0x01, 0x01, 0x7F, 0x01, 0x01,  # T
    0x3F, 0x40, 0x40, 0x40, 0x3F,  # U
    0x1F, 0x20, 0x40, 0x20, 0x1F,  # V
    0x3F, 0x40, 0x38, 0x40, 0x3F,  # W
    0x63, 0x14, 0x08, 0x14, 0x63,  # X
    0x07, 0x08, 0x70, 0x08, 0x07,  # Y
    0x61, 0x51, 0x49, 0x45, 0x43,  # Z
    0x00, 0x7f, 0x41, 0x41, 0x00,  # [
    0x02, 0x04, 0x08, 0x10, 0x20,  # Backslash
    0x00, 0x41, 0x41, 0x7f, 0x00,  # ]
    0x04, 0x02, 0x01, 0x02, 0x04,  # ^
    0x40, 0x40, 0x40, 0x40, 0x40,  # _
    0x00, 0x01, 0x02, 0x04, 0x00,  # `
    0x20, 0x54, 0x54, 0x54, 0x78,  # a
    0x7F, 0x48, 0x44, 0x44, 0x38,  # b
    0x38, 0x44, 0x44, 0x44, 0x20,  # c
    0x38, 0x44, 0x44, 0x48, 0x7F,  # d
    0x38, 0x54, 0x54, 0x54, 0x18,  # e
    0x08, 0x7e, 0x09, 0x01, 0x02,  # f
    0x0c, 0x52, 0x52, 0x52, 0x3e,  # g
    0x7f, 0x08, 0x04, 0x04, 0x78,  # h
    0x00, 0x44, 0x7d, 0x40, 0x00,  # i
    0x20, 0x40, 0x44, 0x3d, 0x00,  # j
    0x7f, 0x10, 0x28, 0x44, 0x00,  # k
    0x00, 0x41, 0x7f, 0x40, 0x00,  # l
    0x7c, 0x04, 0x18, 0x04, 0x78,  # m
    0x7c, 0x08, 0x04, 0x04, 0x78,  # n
    0x38, 0x44, 0x44, 0x44, 0x38,  # o
    0x7c, 0x14, 0x14, 0x14, 0x08,  # p
    0x08, 0x14, 0x14, 0x18, 0x7c,  # q
    0x7c, 0x08, 0x04, 0x04, 0x08,  # r
    0x48, 0x54, 0x54, 0x54, 0x20,  # s
    0x04, 0x3f, 0x44, 0x40, 0x20,  # t
    0x3c, 0x40, 0x40, 0x20, 0x7c,  # u
    0x1c, 0x20, 0x40, 0x20, 0x1c,  # v
    0x3c, 0x40, 0x30, 0x40, 0x3c,  # w
    0x44, 0x28, 0x10, 0x28, 0x44,  # x
    0x0c, 0x50, 0x50, 0x50, 0x3c,  # y
    0x44, 0x64, 0x54, 0x4c, 0x44,  # z
    0x00, 0x08, 0x36, 0x41, 0x00,  # {
    0x00, 0x00, 0x7f, 0x00, 0x00,  # |
    0x00, 0x41, 0x36, 0x08, 0x00,  # }
    0x10, 0x08, 0x08, 0x10, 0x08,  # ~
    0x00, 0x7e, 0x42, 0x42, 0x7e   # \x7f
]


def help():
    print("Help function not finished.")
    time.sleep(2)
    sys.exit(0)


def usage():
    os.system("clear")
    print(sys.argv[0][:-3] + " Usage:\n")
    print(" -d , --debug    Enables the Debugging output.")
    print(" -h , --help     Will print out the help page.")
    print(" -u , --usage    Will print out this page")


def handler(signum, frame):
    Spisend(0x40, 0x0A, 0x00)
    Spisend(0x40, 0x1A, 0x00)
    Spisend(0x42, 0x0A, 0x00)
    Spisend(0x42, 0x1A, 0x00)
    Spisend(0x44, 0x0A, 0x00)
    Spisend(0x44, 0x1A, 0x00)
    Spisend(0x46, 0x0A, 0x00)
    Spisend(0x46, 0x1A, 0x00)
    Spisend(0x40, 0x05, 0x00)
    Spisend(0x42, 0x05, 0x00)
    Spisend(0x44, 0x05, 0x00)
    Spisend(0x46, 0x05, 0x00)
    GPIO.cleanup()
    os.system("clear")
    print("Clean")
    if Failed == 0:
        PSent = "Sent: {0}".format(Sent)
        PRecv = " | Received: {0}".format(Finished)
        PFail = " | Failed: {0}".format(Failed)
        PPerc = " | Failure: 0%"
    else:
        PSent = "Sent: {0}".format(Sent)
        PRecv = " | Received: {0}".format(Finished)
        PFail = " | Failed: {0}".format(Failed)
        PPerc = " | Failure: {0}%".format(str(100 / (Sent / Failed)))
    print(PSent + PRecv + PFail)
    sys.exit()

signal.signal(signal.SIGTSTP, handler)
Debug = False
if len(sys.argv) != 1:

    if sys.argv[1] == "--debug" or sys.argv[1] == "-d":
        Debug = True
    elif sys.argv[1] == "--help" or sys.argv[1] == "-h":
        help()
        sys.exit()
    elif sys.argv[1] == "--usage" or sys.argv[1] == "-u":
        usage()
        sys.exit()
    else:
        usage()
        sys.exit()


def SendValue(Data):
    for i in range(8):
        if (Data & 0x80):
            GPIO.output(10, GPIO.HIGH)
        else:
            GPIO.output(10, GPIO.LOW)
        Data <<= 1
        GPIO.output(11, GPIO.HIGH)
        GPIO.output(11, GPIO.LOW)


def SendSpi(Addr, Reg, Data):
    GPIO.output(8, GPIO.LOW)
    SendValue(Addr | 0x00)
    SendValue(Reg)
    SendValue(Data)
    GPIO.output(8, GPIO.HIGH)


def ReadSpi(Addr, Reg):
    GPIO.output(8, GPIO.LOW)
    SendValue(Addr | 0x01)
    SendValue(Reg)
    Data = 0
    for i in range(8):
        Data <<= 1
        if(GPIO.input(9)):
            Data |= 0x01
        GPIO.output(11, GPIO.HIGH)
        GPIO.output(11, GPIO.LOW)
    GPIO.output(8, GPIO.HIGH)
    return Data


def EightSwitch0():
    try:
        Bus.write_byte_data(0x21, 0x12, 0xFF)
        Read = Bus.read_byte_data(0x21, 0x12)
        if Read == 255:
            Addons["8Switch0"] = True
            Bus.write_byte_data(0x21, 0x12, 0x00)
            if Debug:
                print("Pridopia 8 Switch board MCP23017 detected and loaded!")
    except IOError:
        if Debug:
            print("Pridopia 8 Switch Board not found MCP23017.")


def Led320():
    try:
        Bus.write_byte_data(0x20, 0x12, 0x00)
        Bus.write_byte_data(0x20, 0x03, 0x00)
        Bus.write_byte_data(0x20, 0x00, 0x00)
        Bus.write_byte_data(0x20, 0x01, 0x00)
        Bus.write_byte_data(0x20, 0x0A, 0x00)
        Bus.write_byte_data(0x20, 0x1A, 0x00)
        Bus.write_byte_data(0x20, 0x12, 0x55)
        Bus.write_byte_data(0x20, 0x13, 0xAA)
        Addons["0x20"] = True
    except:
        if Debug:
            print("")

    try:
        Bus.write_byte_data(0x21, 0x12, 0x00)
        Bus.write_byte_data(0x21, 0x03, 0x00)
        Bus.write_byte_data(0x21, 0x00, 0x00)
        Bus.write_byte_data(0x21, 0x01, 0x00)
        Bus.write_byte_data(0x21, 0x0A, 0x00)
        Bus.write_byte_data(0x21, 0x1A, 0x00)
        Bus.write_byte_data(0x21, 0x12, 0x55)
        Bus.write_byte_data(0x21, 0x13, 0xAA)
        Addons["0x21"] = True
    except:
        if Debug:
            print("")

    try:
        Bus.write_byte_data(0x22, 0x12, 0x00)
        Bus.write_byte_data(0x22, 0x00, 0x00)
        Bus.write_byte_data(0x22, 0x01, 0x00)
        Bus.write_byte_data(0x22, 0x0A, 0x00)
        Bus.write_byte_data(0x22, 0x1A, 0x00)
        Bus.write_byte_data(0x22, 0x12, 0x55)
        Bus.write_byte_data(0x22, 0x13, 0xAA)
        Addons["0x22"] = True
    except:
        if Debug:
            print("")

    try:
        Read11 = Bus.read_byte_data(0x21, 0x12)
        Read12 = Bus.read_byte_data(0x21, 0x13)
        Read21 = Bus.read_byte_data(0x22, 0x12)
        Read22 = Bus.read_byte_data(0x22, 0x13)
        if (
            Read11 == 85 and Read21 == 85 and
            Read12 == 0xAA and Read22 == 0xAA
        ):
            Addons["Led320"] = True
            if Debug:
                print("")
        else:
            if Debug:
                print("")

        Bus.write_byte_data(0x21, 0x00, 0xFF)
        Bus.write_byte_data(0x22, 0x00, 0xFF)
        Bus.write_byte_data(0x21, 0x12, 0x00)
        Bus.write_byte_data(0x21, 0x13, 0x00)
        Bus.write_byte_data(0x22, 0x12, 0x00)
        Bus.write_byte_data(0x22, 0x13, 0x00)
    except IOError:
        if Debug:
            print("")


def Measure(ECHO, TRIG):
    stop = start = 0
    GPIO.setup(TRIG, GPIO.OUT)
    GPIO.setup(ECHO, GPIO.IN)
    MaxTime = 0.05
    GPIO.output(TRIG, 1)
    time.sleep(0.00001)
    GPIO.output(TRIG, 0)
    start = time.time()
    timeout = start + MaxTime
    while GPIO.input(ECHO) == 0 and start <= timeout:
        start = time.time()
    else:
        if time.time() > timeout:
            stop = start = 1

    while GPIO.input(ECHO) and time.time() <= timeout:
        stop = time.time()
    else:
        if time.time() > timeout:
            stop = start = 1

    GPIO.setup(TRIG, GPIO.OUT)
    GPIO.output(TRIG, False)

    Elap = stop - start
    Distance = (Elap * 34300) / 2.0
    time.sleep(0.02)

    return Distance


def EightSwitchS():
    try:
        SendSpi(0x40, 0x0A, 0x00)

        SendSpi(0x40, 0x0A, 0x55)
        Read = ReadSpi(0x40, 0x0A)
        if Read == 0x55:
            Addons["8SwitchS"] = True
            SendSpi(0x40, 0x0A, 0x00)
    except:
        Addons["8SwitchS"] = False


def Led32S():
    try:
        SendSpi(0x40, 0x0A, 0x00)
        SendSpi(0x42, 0x0A, 0x00)
        SendSpi(0x40, 0x0A, 0x55)
        SendSpi(0x42, 0x0A, 0x55)
        Read1 = ReadSpi(0x40, 0x0A)
        Read2 = ReadSpi(0x42, 0x0A)
        if Read1 == 0x55 and Read2 == 0x55:
            Addons["Led32S"] = True
            SendSpi(0x40, 0x0A, 0x00)
            SendSpi(0x42, 0x0A, 0x00)
        else:
            SendSpi(0x40, 0x0A, 0x00)
            SendSpi(0x40, 0x1A, 0x00)
            SendSpi(0x42, 0x0A, 0x00)
            SendSpi(0x42, 0x1A, 0x00)
    except:
        if Debug:
            print("")


def ServoTest():
    try:
        pwm = PWM(0x41, debug=Debug)
        pwm.setPWMFreq(60)
        Addons["Servo"] = True
        pwm2 = PWM(0x40, debug=Debug)
        pwm2.setPWMFreq(60)
        Addons["2Servo"] = True
    except:
        print "Failed to enabled Servo Board."


def RtcTest():
    try:
        Bus.read_byte_data(0x68, 0x00)        # To wake it up if is sleeping.
        Read = Bus.read_byte_data(0x68, 0x00)
        if Read != 0x00:
            Addons["RTC"] = True
    except:
        Addons["RTC"] = False


def BmpTest():
    try:
        Bmp = BMP85(0x77)
        Pressure = Bmp.readPressure()
        if Debug:
            print "Pressure: " + Pressure
        if Pressure > 15000:
            Addons["BMP085"] = True
    except:
        pass


def Temp():
    try:
        Data = Bus.read_i2c_block_data(0x48, 0)
        Da = Data[0]
        Db = Data[1]
        Temp = ((((Da << 8) | Db) >> 4) * 0.065)
        Addons["TMP102"] = True
        return Temp
    except:
        Addons["TMP102"] = False

def LEDMatrix():
    try:
        Addr1 = Bus.read_byte_data(0x70, 0x00)
        Addr2 = Bus.read_byte_data(0x71, 0x00)
        if Addr1 != 0x00 and Addr2 != 0x00:
            Addons["16x16Matrix"] = True
        Addr3 = Bus.read_byte_data(0x72, 0x00)
        if Addr1 != 0x00 and Addr2 != 0x00 and Addr3 != 0x00:
            Addons["16x16Matrix"] = False
            Addons["24x16Matrix"] = True
    except:
        Addons["24x16Matrix"] = False


def SpiButton():
    try:
        SendSpi(0x40, 0x13, 0xFF)
        Read = ReadSpi(0x40, 0x13)
        if Read != 0xFF:
            Addons["SpiButton"] = True
    except:
        Addons["SpiButton"] = False


def I2CButton():
    try:
        Bus.write_byte_data(0x21, 0x13, 0x00)
        Bus.write_byte_data(0x21, 0x13, 0x55)
        time.sleep(0.1)
        Read1 = Bus.read_byte_data(0x21, 0x13)
        if Read1 != 0x55:
            Addons["I2CButton"] = True
            Bus.write_byte_data(0x21, 0x01, 0xFF)   # Set all BankB to Inputs
            Bus.write_byte_data(0x21, 0x03, 0xFF)   # Invert Polarity of pins
            Bus.write_byte_data(0x21, 0x00, 0x00)   # Set all BankA to Outputs
        else:
            Addons["I2CButton"] = False
    except:
        Addons["I2CButton"] = False


def read_B18_Raw():
    os.system('modprobe w1-gpio')
    os.system('modprobe w1-therm')

    base_dir = '/sys/bus/w1/devices/'
    device_folder = glob.glob(base_dir + '28*')[0]
    device_file = device_folder + '/w1_slave'
    f = open(device_file, 'r')
    lines = f.readlines()
    f.close()
    return lines


def read_B18():
    lines = read_B18_Raw()
    while lines[0].strip()[-3:] != 'YES':
            time.sleep(0.2)
            lines = read_B18_Raw()
    equals_pos = lines[1].find('t=')
    if equals_pos != -1:
        temp_string = lines[1][equals_pos+2:]
        temp_c = float(temp_string) / 1000.0
        temp_f = temp_c * 9.0 / 5.0 + 32.0
        sendCmd("sensor-update Temp18B20 %.2f" % (temp_c))


def Check():
    if Connected == "False":
        EightSwitch0()
        Led320()
        EightSwitchS()
        Led32S()
        ServoTest()
        RtcTest()
        BmpTest()
        AdsTest()
        LEDMatrix()
        Temp()
        SpiButton()
        I2CButton()
        Finished = 0
        Sent = 0
        Failed = 0


def stringbit(a):
    Str = bin(a)[2:]
    return '0'*(8-len(Str)) + Str


def SpisendValue(value):
    GPIO.setup(10, GPIO.OUT)
    GPIO.setup(11, GPIO.OUT)
    for i in range(8):
        if (value & 0x80):
            GPIO.output(10, GPIO.HIGH)
        else:
            GPIO.output(10, GPIO.LOW)
        GPIO.output(11, GPIO.HIGH)
        GPIO.output(11, GPIO.LOW)
        value <<= 1


def Spisend(opcode, addr, data):
    GPIO.setup(8, GPIO.OUT)
    GPIO.output(8, GPIO.LOW)
    SpisendValue(opcode | 0x00)
    SpisendValue(addr)
    SpisendValue(data)
    GPIO.output(8, GPIO.HIGH)


def Spiread(opcode, addr):
    GPIO.output(8, GPIO.LOW)
    SpisendValue(opcode | 0x01)
    SpisendValue(addr)
    value = 0
    for i in range(8):
        value <<= 1
        if(GPIO.input(9)):
            value |= 0x01
        GPIO.output(11, GPIO.HIGH)
        GPIO.output(11, GPIO.LOW)
    GPIO.output(8, GPIO.HIGH)
    return value


def Write(Addr, Reg, Bit):
    Bus.write_byte_data(Addr, Reg, Bit)
    return 1


def SetV1(Input):
    Val = int((float(Input) * 256) / 2.048)
    Val = bin(Val)[2:]
    Length = len(Val)
    Add = 8 - Length
    Val = "0" * Add+Val
    BinA = Val[:4]
    BinB = Val[3:]
    print(BinA, " : ", BinB)
    BinA = int("0011" + BinA, 2)
    BinB = int(BinB + "0000", 2)
    spi.xfer2([BinA, BinB])
    #return True


def SetV2(Input):
    Val = int((float(Input) * 256) / 2.048)
    Val = bin(Val)[2:]
    Length = len(Val)
    Add = 8 - Length
    Val = "0"*Add+Val
    BinA = Val[:4]
    BinB = Val[3:]
    print(BinA, " : ", BinB)
    BinA = int("1011" + BinA, 2)
    BinB = int(BinB + "0000", 2)
    print BinA, BinB
    spi.xfer2([BinA, BinB])


Commands = [""] * 10
GpioS = [False] * 35
GpioDir = ["OUT"] * 35

GPIO_ECHO = 0
GPIO_TRIGGER = 0
UltraSonic = "UltraSonic"
Displayed = False
lastX = 0
lastY = 0
SCLK = 24
DIN = 23
DC = 22
RST = 17
CE = 21
LIGHT = 18

GPIO.setmode(GPIO.BCM)
GPIO.setup(0, GPIO.OUT)
GPIO.setup(9, GPIO.OUT)

GPIO.setup(DIN, GPIO.OUT)
GPIO.setup(SCLK, GPIO.OUT)
GPIO.setup(DC, GPIO.OUT)
GPIO.setup(RST, GPIO.OUT)
GPIO.setup(CE, GPIO.OUT)
GPIO.setup(LIGHT, GPIO.OUT)

setup = True

Delay = 300
News = []
Feed = 'feeds.bbci.co.uk/news/rss.xml?edition=uk'


def Update_News(Url):
    Args = ['curl', Url]
    Proc = Call(Args, stdout=subprocess.PIPE)
    Out = Proc.stdout.read()
    Find = re.findall("<title>([\S ]*)</title>", Out)
    Find.pop(0)
    Find.pop(0)

    for A in Find:
        if (
            A[:7] == "VIDEO: " or
            A[:53] == "AUD" or
            A[:4] == "In p" or
            A[:4] == "Day "
        ):
            A = False
        else:
            News.append(A)
    return News


def sendCmd(cmd):
    n = len(cmd)
    a = array('c')
    a.append(chr((n >> 24) & 0xFF))
    a.append(chr((n >> 16) & 0xFF))
    a.append(chr((n >> 8) & 0xFF))
    a.append(chr(n & 0xFF))
    Socket.send(a.tostring() + cmd)


def SetAngle(Channel, Addr, Angle, Mode):

    if Angle >= SMax:
        Angle = SMax
    elif Angle <= SMin:
        Angle = SMin

    print Channel, Addr, Angle, Mode
    if Mode == "Add":

        if Debug:
            print("Setting Angle...")
            print(Addr, Channel, Angle)
        if Addr == 0x41:
            Channel = int(Channel)
            Angle = int(Angle)
            pwm.setPWM(Channel, 0, Angle)
        elif Addr == 0x40:
            Channel = int(Channel)
            Angle = int(Angle)
            pwm2.setPWM(Channel, 0, Angle)

    elif Mode == "Set":

        if Addr == 0x41:
            if Debug:
                print("Addr 0x41, Angle {0}".format(Angle))
            pwm.setPWM(Channel, 0, Angle)
            SAngles[Channel] = Angle
            SAngle = Angle
        elif Addr == 0x40:
            if Debug:
                print("Addr 0x40, Angle {0}".format(Angle))
            pwm2.setPWM(Channel, 0, Angle)
            SAngles2[Channel] = Angle
        else:
            print("Address does not match..")

    else:
        if Debug:
            print("Error: Mode not set!")

    sendCmd("sensor-update Servo-{0} {1}".format(Channel, Angle))


def changechannel(address, adcConfig):
    tmp = Bus.write_byte(address, adcConfig)


def getadcreading(address, adcConfig):
    adcreading = Bus.read_i2c_block_data(address, adcConfig)
    h = adcreading[0]
    m = adcreading[1]
    l = adcreading[2]
    s = adcreading[3]
    while (s & 128):
        adcreading = Bus.read_i2c_block_data(address, adcConfig)
        h = adcreading[0]
        m = adcreading[1]
        l = adcreading[2]
        s = adcreading[3]

    t = ((h & 0b00000001) << 16) | (m << 8) | l
    if (h > 128):
        t = ~(0x020000 - t)
    return (t / 64)


def GoToXY8544(X, Y):
    lastX = X
    lastY = Y
    lcd_cmd8544(X + 128)
    lcd_cmd8544(Y + 64)


def lcd_text8544(Text):
    for i in range(len(Text)):
        lcd_display_char8544(Text[i])


def lcd_cls8544():
    GoToXY8544(0, 0)
    for i in range(84):
        for j in range(6):
            lcd_data8544(0)


def lcd_display_char8544(Char):
    index = (ord(Char) - 32) * 5
    if ord(Char) >= 32 and ord(Char) <= 127:
        for i in range(5):
            lcd_data8544(font[index+i])
        lcd_data8544(0)
    elif ord(Char) == 32:
        lcd_data8544(0)
        lcd_data8544(0)
        lcd_data8544(0)
        lcd_data8544(0)
        lcd_data8544(0)
        lcd_data8544(0)


def lcd_begin8544(Contrast):
    GPIO.output(RST, False)
    time.sleep(0.100)
    GPIO.output(RST, True)
    lcd_cmd8544(0x21)
    lcd_cmd8544(0x14)
    lcd_cmd8544(Contrast)
    lcd_cmd8544(0x20)
    lcd_cmd8544(0xc)
    lcd_cls8544()


def SPI8544(c):
    for i in xrange(8):
        GPIO.output(DIN, (c & (1 << (7-i))) > 0)
        GPIO.output(SCLK, True)
        GPIO.output(SCLK, False)


def lcd_cmd8544(c):
    GPIO.output(DC, False)
    SPI8544(c)


def lcd_data8544(c):
    GPIO.output(DC, True)
    SPI8544(c)

def GetRam():
    Proc = Popen("free", stdout=PIPE, shell=True)
    Read = "True"
    Lines = []
    while Read:
        Read = Proc.stdout.readline()
        Lines.append(Read)
    Ram = Lines[1]
    Ram = Ram.split()
    Total = float(Ram[1])/1024.0
    Used = float(Ram[3])/1024.0
    Usage = 100.0 / (Total / Used)
    Total = "%.2f" % Total
    Used = "%.2f" % Used
    Usage = "%.2f" % Usage
    return (Total, Used, Usage)

def GetWifi():
    Proc = Popen("iwconfig", stdout=PIPE, shell=True, stderr=open("/dev/null"))
    Line = True
    Lines = []
    SSID = ""
    Signal = ""
    Link = ""
    while Line:
        Line = Proc.stdout.readline()
        Lines.append(Line)
        if not SSID:
            SSID = re.findall('ESSID:"([\S ]*)"',Line)
        if not Signal:
            Signal = re.findall('Signal level=[-]?([0-9]*) dBm',Line)
        if not Link:
            Link = re.findall('Quality=([0-9]*/[0-9]*)', Line)
	if not SSID:
		SSID = ["NoSSID"]
	if not Signal:
		Signal = ["NoSignal"]
	if not Link:
		Link = ["NoLink"]
    return SSID[0], Signal[0], Link[0]

def LPrint(Text):
    if type(Text) != str:
        Text = Text[0]
    Text = Text[6:]
    Text = binascii.hexlify(str(Text))
    Args = ["sudo", "python", "lib/Matrix/led_print.py", Text]
    Popen(Args)


def SPrint(Text):
    if type(Text) != str:
        Text = Text[0]
    print Text
    Text = Text[6:]
    print Text
    Args = ["sudo", "python", "lib/Matrix/led_print_16.py", Text]
    Popen(Args)

def TestIP(IP):
    IP = IP.split(".")
    for A in IP:
        try:
            int(A)
        except:
            return False
    return True

def SMatrix(Text):
    global STimePID
    global SLedMessagePID
    DNull = open('/dev/null', 'w')
    Texttmp = Text[4:-1]
    Text = Text.upper()
    Text = Text[4:-1]
    if Text == "TIME" and STimePID == 9999:
        Args = ["sudo", "python", "lib/Matrix/led_16x16_small.py"]
        STime = Popen(Args, stdout=DNull, stderr=DNull)
        STimePID = STime.pid
    elif Text == "TIMESTOP" and STimePID != 9999:
        os.kill(STimePID, signal.SIGINT)
        STimePID = 9999
    elif Text == "TIMESTOP" and STimePID == 9999:
        print "No Process running 16x16 Time."
    elif Text == "IMG":
        Args = ["sudo", "python", "lib/Matrix/led_single_img_16.py"]
        Popen(Args, stdout=DNull, stderr=DNull)
    elif Text == "CLS":
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_clear.py"
        ]
        Popen(Args, stdout=DNull, stderr=DNull)
    elif Text == "STOP":
        if STimePID != 9999:
            os.kill(STimePID, signal.SIGINT)
            STimePID = 9999
        if SLedMessagePID != 9999:
            os.kill(SLedMessagePID, signal.SIGINT)
            SLedMessagePID = 9999
    elif Text == "RAM":
        Total, Used, Usage = GetRam()
        Text = "RAM: {}/{}MB".format(Used, Total)
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_finished.py",
            Text
        ]
        if SLedMessagePID == 9999:
            Message = Popen(Args, stdout=DNull, stderr=DNull)
            SLedMessagePID = Message.pid
    elif Text == "WIFI IP":
        cmd = [
            "ip addr show wlan0 | grep inet | awk '{print $2}' | cut -d/ -f1"
        ]
        Run = Popen(cmd, shell=True, stdout=PIPE)
        IP = Run.stdout.read()
        if TestIP(IP):
            Text = "IP:{} ".format(IP)        # IP is all numbers seperated by a .
        else:
            Text = "IP:{}-- ".format(IP)    # IP is not true format! Ignore and still print ( editable... ) continue to exit command.
            print "Possible fake IP / Incorrect IP / Not set up WiFi?"
        Text = "IP:{} ".format(IP)
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_finished.py",
            Text
        ]
        if SLedMessagePID == 9999:
            Message = Popen(Args, stdout=DNull, stderr=DNull)
            SLedMessagePID = Message.pid
    elif Text == "WIFI SSID":
        SSID, Signal, Link = GetWifi()
        Text = "SSID:{} ".format(SSID)
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_finished.py",
            Text
        ]
        if SLedMessagePID == 9999:
            Message = Popen(Args, stdout=DNull, stderr=DNull)
            SLedMessagePID = Message.pid
    elif Text == "WIFI SIG":
        SSID, Signal, Link = GetWifi()
        Text = "SIG:{}/70 ".format(Signal)
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_finished.py",
            Text
        ]
        if SLedMessagePID == 9999:
            Message = Popen(Args, stdout=DNull, stderr=DNull)
            SLedMessagePID = Message.pid
    else:
        Text = Texttmp.replace(' ', '%20%')
        print Text
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_finished.py",
            Text
        ]
        if SLedMessagePID == 9999:
            Message = Popen(Args, stdout=DNull, stderr=DNull)
            SLedMessagePID = Message.pid

def Matrix(Text):
    global TimePID
    global LedMessagePID
    DNull = open('/dev/null', 'w')
    Text = Text[0]
    Texttmp = Text[3:-1]
    Text = Text.upper()
    Text = Text[3:-1]
    if Text == "TIME" and TimePID == 9999:
        Args = ["sudo", "python", "lib/Matrix/led_single.py"]
        Time = Popen(Args, stdout=DNull, stderr=DNull)
        TimePID = Time.pid
    elif Text == "TIME" and TimePID != 9999:
        os.kill(TimePID, signal.SIGINT)
        TimePID = 9999
        Args = ["sudo", "python", "lib/Matrix/led_single.py"]
        Time = Popen(Args, stdout=DNull, stderr=DNull)
        TimePID = Time.pid
    elif Text == "TIMESTOP" and TimePID != 9999:
        os.kill(TimePID, signal.SIGINT)
        TimePID = 9999
    elif Text == "IMG":
        Args = [
            "sudo", "python",
            "lib/Matrix/led_single_img.py"
        ]
        Popen(Args, stdout=DNull, stderr=DNull)
    elif Text == "CLS":
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_clear.py"
        ]
        Popen(Args, stdout=DNull, stderr=DNull)
    elif Text == "STOP" or Text == "KILL" or Text == "OFF":
        if TimePID != 9999:
            os.kill(TimePID, signal.SIGINT)
            TimePID = 9999
        if LedMessagePID != 9999:
            os.kill(LedMessagePID, signal.SIGINT)
            LedMessagePID = 9999
    elif Text == "RAM":
        Total, Used, Usage = GetRam()
        Text = "RAM:{}/{}MB ".format(Used, Total)
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_finished.py",
            Text
        ]
        if LedMessagePID == 9999:
            Message = Popen(Args, stdout=DNull, stderr=DNull)
            LedMessagePID = Message.pid
    elif Text == "WIFI IP":
        cmd = [
            "ip addr show wlan0 | grep inet | awk '{print $2}' | cut -d/ -f1"
        ]
        Run = Popen(cmd, shell=True, stdout=PIPE)
        IP = Run.stdout.read()
        if TestIP(IP):
            Text = "IP:{} ".format(IP)        # IP is all numbers seperated by a . if not then this test fails. ( Characters / Symbols will fail it )
        else:
            Text = "No WiFi ".format(IP)    # IP is not true format! print No WiFi ( Remove whole if statement if false negative )
            print "Possible fake IP / Incorrect IP / Not set up WiFi?"
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_finished.py",
            Text
        ]
        if LedMessagePID == 9999:
            Message = Popen(Args, stdout=DNull, stderr=DNull)
            LedMessagePID = Message.pid
    elif Text == "WIFI SSID":
        SSID, Signal, Link = GetWifi()
        Text = "SSID:{} ".format(SSID)
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_finished.py",
            Text
        ]
        if LedMessagePID == 9999:
            Message = Popen(Args, stdout=DNull, stderr=DNull)
            LedMessagePID = Message.pid
    elif Text == "WIFI SIG":
        SSID, Signal, Link = GetWifi()
        Text = "SIG:{}/70 ".format(Signal)
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_finished.py",
            Text
        ]
        if LedMessagePID == 9999:
            Message = Popen(Args, stdout=DNull, stderr=DNull)
            LedMessagePID = Message.pid
    else:
        Text = Texttmp.replace(' ', '%20%')
        print Text
        Args = [
            "sudo",
            "python",
            "lib/Matrix/led_finished.py",
            Text
        ]
        if LedMessagePID == 9999:
            Message = Popen(Args, stdout=DNull, stderr=DNull)
            LedMessagePID = Message.pid

SSID, Signal, Link = GetWifi()
print "Wireless settings > Signal: {}    SSID: {}     Link: {}".format(Signal, SSID, Link)
def Stepper_Forward_A(Delay, Ang):

    Pattern = [
        0b0001,
        0b0011,
        0b0110,
        0b1100,
        0b1001
    ]

    for A in range(len(Pattern)):
        Pattern[A] = Pattern[A] << 4

    GPIO.setup(17, 0)
    GPIO.setup(18, 0)
    GPIO.setup(27, 0)
    GPIO.setup(22, 0)
    while Ang != 0 and Ang > 0:
        for A in range(len(Pattern)):
            Cur = '{:08b}'.format(Pattern[A])
            P1 = int(Cur[:1])
            P2 = int(Cur[1:2])
            P3 = int(Cur[2:3])
            P4 = int(Cur[3:4])
            GPIO.output(17, P1)
            GPIO.output(18, P2)
            GPIO.output(27, P3)
            GPIO.output(22, P4)
            time.sleep(Delay)
        Ang -= 1

    GPIO.output(17, 0)
    GPIO.output(18, 0)
    GPIO.output(27, 0)
    GPIO.output(22, 0)


def Stepper_Forward_B(Delay, Ang):

    Pattern = [
        0b0001,
        0b0011,
        0b0110,
        0b1100,
        0b1001
    ]

    GPIO.setup(23, 0)
    GPIO.setup(24, 0)
    GPIO.setup(25, 0)
    GPIO.setup(4, 0)
    while Ang != 0 and Ang > 0:
        for A in range(len(Pattern)):
            Cur = '{:08b}'.format(Pattern[A])
            Cur = Cur[4:]
            P1 = int(Cur[:1])
            P2 = int(Cur[1:2])
            P3 = int(Cur[2:3])
            P4 = int(Cur[3:4])
            GPIO.output(23, P1)
            GPIO.output(24, P2)
            GPIO.output(25, P3)
            GPIO.output(4, P4)
            time.sleep(Delay)
        Ang -= 1

    GPIO.output(23, 0)
    GPIO.output(24, 0)
    GPIO.output(25, 0)
    GPIO.output(4, 0)


def Stepper_Reverse_A(Delay, Ang):

    Pattern = [
        0b1000,
        0b1100,
        0b0110,
        0b0011,
        0b1001
    ]

    Ang = Ang * -1
    for A in range(len(Pattern)):
        Pattern[A] = Pattern[A] << 4


    GPIO.setup(17, 0)
    GPIO.setup(18, 0)
    GPIO.setup(27, 0)
    GPIO.setup(22, 0)
    while Ang != 0 and Ang > 0:
        for A in range(len(Pattern)):
            Cur = '{:08b}'.format(Pattern[A])
            P1 = int(Cur[:1])
            P2 = int(Cur[1:2])
            P3 = int(Cur[2:3])
            P4 = int(Cur[3:4])
            GPIO.output(17, P1)
            GPIO.output(18, P2)
            GPIO.output(27, P3)
            GPIO.output(22, P4)

            time.sleep(Delay)
        Ang -= 1

    GPIO.output(17, 0)
    GPIO.output(18, 0)
    GPIO.output(27, 0)
    GPIO.output(22, 0)


def Stepper_Reverse_B(Delay, Ang):

    Pattern = [
        0b1000,
        0b1100,
        0b0110,
        0b0011,
        0b1001
    ]

    Ang = Ang * -1

    GPIO.setup(23, 0)
    GPIO.setup(24, 0)
    GPIO.setup(25, 0)
    GPIO.setup(4, 0)
    while Ang != 0 and Ang > 0:
        for A in range(len(Pattern)):
            Cur = '{:08b}'.format(Pattern[A])
            Cur = Cur[4:]
            P1 = int(Cur[:1])
            P2 = int(Cur[1:2])
            P3 = int(Cur[2:3])
            P4 = int(Cur[3:4])
            GPIO.output(23, P1)
            GPIO.output(24, P2)
            GPIO.output(25, P3)
            GPIO.output(4, P4)
            time.sleep(Delay)
        Ang -= 1

    GPIO.output(23, 0)
    GPIO.output(24, 0)
    GPIO.output(25, 0)
    GPIO.output(4, 0)
    
    
def I2CSetFunc():

    Address = [0x20, 0x21, 0x22, 0x23, 0x24, 0x25, 0x26, 0x27]
    for Addr in Address:
        try:
            Write(Addr, 0x00, 0x00)
            Write(Addr, 0x01, 0x00)
            Write(Addr, 0x0A, 0x00)
        except:
            print str(Addr) + " not availible."
            
def GetDHT(Pin):
    Proc = Popen(["python", "lib/Temp.py", Pin], stdout=PIPE)
    Read = True
    Find = False
    while Read:
        Read = Proc.stdout.readlines()
        Read = Read[0]
        Read = Read[:-1]
        if not Find:
            Find = re.findall(r'[[0-9]*[.]+[0-9]* C,[0-9]*[.]+[0-9]* %]', Read)
            if Read == "[0 C,0 %]":
                print "No Data."
                return "0","0"
            if Find:
                Find = Find[0]
                Read = False
                Find = Find.replace("[", "")
                Find = Find.replace("]", "")
                Find = Find.replace("%", "")
                Find = Find.replace("C", "")
                Find = Find.replace(" ", "")
                Find = Find.split(",")
                print "{0}C, {1}%".format(Find[0], Find[1])
                return Find[0],Find[1]
    return

Day = time.strftime("%d")
Day2 = time.strftime("%A")
Month = time.strftime("%B")
Year = time.strftime("%Y")
Date = time.strftime("%A")

SpiSetup = False
RSS1602FeedPID = 9999
RSS2004FeedPID = 9999
RSS8544FeedPID = 9999
SPInputs = []
I2Inputs = []
lcd_cls8544()
lcd_begin8544(0xA5)
GPIO_ECHO = 0
GPIO_TRIGGER = 0
I2CSetup = False

os.system("clear")
print("Connecting ...")
Pins = [17, 18, 27, 22, 23, 24, 25, 4, 28, 29, 30, 31]
for A in Pins:
    GPIO.setup(A, GPIO.OUT)
    GPIO.output(A, False)

def Connect():
    Check()
    global Socket
    try:
        Setup = False
        Socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        Socket.connect((HOST, PORT))
        print("Connected!")
        Connected = True
        sendCmd("broadcast startup")
        return
    except:
        print("Scratch not up. Sleeping for 5 and trying again.")
        time.sleep(5)
        Connected = False
        Connect()
if not Connected:
    Connect()


def List():
    for Key, Value in Addons.iteritems():
        print(Key)

List()


GPIO.setup(11, GPIO.OUT)
GPIO.setup(10, GPIO.OUT)
GPIO.setup(9, GPIO.IN)
GPIO.setup(8, GPIO.OUT)
Spisend(0x40, 0x0A, 0x00)
Spisend(0x40, 0x1A, 0x00)
Spisend(0x42, 0x0A, 0x00)
Spisend(0x42, 0x1A, 0x00)
Spisend(0x44, 0x0A, 0x00)
Spisend(0x44, 0x1A, 0x00)
Spisend(0x46, 0x0A, 0x00)
Spisend(0x46, 0x1A, 0x00)
Spisend(0x40, 0x05, 0x00)
Spisend(0x42, 0x05, 0x00)
Spisend(0x44, 0x05, 0x00)
Spisend(0x46, 0x05, 0x00)
Spisend(0x40, 0x00, 0x00)
Spisend(0x40, 0x01, 0x00)
Spisend(0x40, 0x12, 0x00)
Spisend(0x40, 0x13, 0x00)
Spisend(0x40, 0x0A, 0xFF)
Spisend(0x40, 0x1A, 0xFF)
Spisend(0x40, 0x0A, 0x00)
Spisend(0x40, 0x1A, 0x00)

Spisend(0x48, 0x0A, 0x00)
Spisend(0x48, 0x1A, 0x00)
Spisend(0x4A, 0x0A, 0x00)
Spisend(0x4A, 0x1A, 0x00)
Spisend(0x4C, 0x0A, 0x00)
Spisend(0x4C, 0x1A, 0x00)
Spisend(0x4E, 0x0A, 0x00)
Spisend(0x4E, 0x1A, 0x00)
Spisend(0x48, 0x05, 0x00)
Spisend(0x4A, 0x05, 0x00)
Spisend(0x4C, 0x05, 0x00)
Spisend(0x4E, 0x05, 0x00)
Spisend(0x48, 0x00, 0x00)
Spisend(0x48, 0x01, 0x00)
Spisend(0x48, 0x12, 0x00)
Spisend(0x48, 0x13, 0x00)
Spisend(0x48, 0x0A, 0xFF)
Spisend(0x48, 0x1A, 0xFF)
Spisend(0x48, 0x0A, 0x00)
Spisend(0x48, 0x1A, 0x00)
try:
    Bus.write_byte_data(0x21, 0x00, 0xFF)
    Bus.write_byte_data(0x21, 0x01, 0xFF)
except:
    print("")


GPIO.output(17, False)
GPIO.output(22, False)
GPIO.setup(23, GPIO.IN)
LastMsg = time.time()

while True:
    try:
        if not Setup:
            try:
                print("")
                Bus.write_byte_data(0x21, 0x00, 0xFF)
                Bus.write_byte_data(0x21, 0x01, 0xFF)
                Bus.write_byte_data(0x21, 0x12, 0x55)
                Bus.write_byte_data(0x21, 0x12, 0x00)

                Bus.write_byte_data(0x22, 0x00, 0xFF)
                Bus.write_byte_data(0x22, 0x01, 0xFF)
                Bus.write_byte_data(0x22, 0x12, 0x55)
                Bus.write_byte_data(0x22, 0x12, 0x00)

                Bus.write_byte_data(0x23, 0x00, 0xFF)
                Bus.write_byte_data(0x23, 0x01, 0xFF)
                Bus.write_byte_data(0x23, 0x12, 0x55)
                Bus.write_byte_data(0x23, 0x12, 0x00)

                Bus.write_byte_data(0x20, 0x00, 0xFF)
                Bus.write_byte_data(0x20, 0x01, 0xFF)
                Bus.write_byte_data(0x20, 0x12, 0x55)
                Bus.write_byte_data(0x20, 0x12, 0x00)
            except:
                if Debug:
                    print("")
            Setup = True
        GPIO.setmode(GPIO.BCM)
        Data = Socket.recv(1024)
        if not Data:
            if (time.time() - LastMsg) > 30:
                Socket.close()
                time.sleep(1)
                Connect()
                print "Reconnected."
            time.sleep(0.5)
        if Data:
            LastMsg = time.time()
            Data = Data.replace('"', ' ')
            Data = Data.replace("'", ' ')
            Data = Data.replace('BROADCAST', '')
            Data = Data.replace('broadcast', '')
            Data = Data[5:]

            Data = filter(lambda x: x in string.printable, Data)
            if Debug:
                print "Data> " + Data

            Lcd8544cls = Lcd8544bl = Lcd8544wip = Lcd8544eip = False
            Lcd8544c = Lcd8544date = Lcd8544time = Lcd8544ram = Lcd8544wifi = False
            Lcd85442 = Lcd8544 = Lcd8544rss = Lcd8544rssc = False
            Lcd1602cls = Lcd1602bl = Lcd1602wip = Lcd1602eip = False
            Lcd1602c = Lcd16022 = Lcd1602date = Lcd1602time = False
            Lcd1602 = Lcd1602ram = Lcd1602wifi = Lcd1602rss = Lcd1602rssc = False
            Lcd1602clsg = Lcd1602blg = Lcd1602wipg = Lcd1602eipg = False
            Lcd1602cg = Lcd16022g = Lcd1602dateg = Lcd1602timeg = False
            Lcd1602g = Lcd1602rssg = Lcd1602rsscg = Lcd1602ramg = Lcd1602wifig = False
            Lcd2004cls = Lcd2004bl = Lcd2004wip = Lcd2004eip = False
            Lcd2004c = Lcd20042 = Lcd2004date = Lcd2004time = False
            Lcd2004 = Lcd2004rssc = Lcd2004rss = Lcd2004ram = Lcd2004wifi = False
            Lcd2004clsg = Lcd2004blg = Lcd2004wipg = Lcd2004eipg = False
            Lcd2004cg = Lcd20042g = Lcd2004dateg = Lcd2004timeg = False
            Lcd2004g = Lcd2004rssg = Lcd2004rsscg = Lcd2004ramg = Lcd2004wifig = False

            if Addons["lcd8544"]:
                Lcd8544cls = re.findall(
                    r'LCD8544 cls', Data)
                Lcd8544rss = re.findall(
                    r'LCD8544 rss ([0-9]+)', Data)
                Lcd8544bl = re.findall(
                    r'LCD8544 bl (on|off)', Data)
                Lcd8544rssc = re.findall(
                    r'LCD8544 rss (off)', Data)
                Lcd8544wip = re.findall(
                    r'LCD8544 ([0-9]*),([0-9]*) wip', Data)
                Lcd8544eip = re.findall(
                    r'LCD8544 ([0-9]*),([0-9]*) eip', Data)
                Lcd8544date = re.findall(
                    r'LCD8544 ([0-9]*),([0-9]*) date', Data)
                Lcd8544time = re.findall(
                    r'LCD8544 ([0-9]*),([0-9]*) time', Data)
                Lcd8544ram = re.findall(
                    r'LCD8544 ([0-9]*),([0-9]*) ram', Data)
                Lcd8544wifi = re.findall(
                    r'LCD8544 ([0-9]*),([0-9]*) wifi ([\S]*)', Data)
                Lcd8544c = re.findall(
                    r'LCD8544 con ([0-9]*)', Data)
                Lcd85442 = re.findall(
                    r'LCD8544 ([\S ]*)', Data)
                Lcd8544 = re.findall(
                    r'LCD8544 ([0-9]*),([0-9]*) ([\S ]*)', Data)

            if Addons["lcd1602"]:
                Lcd1602cls = re.findall(
                    r'LCD1602 cls', Data)
                Lcd1602rss = re.findall(
                    r'LCD1602 rss ([0-9]+)', Data)
                Lcd1602bl = re.findall(
                    r'LCD1602 bl (on|off)', Data)
                Lcd1602rssc = re.findall(
                    r'LCD1602 rss off', Data)
                Lcd1602wip = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) wip', Data)
                Lcd1602eip = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) eip', Data)
                Lcd16022 = re.findall(
                    r'LCD1602 ([\S ]*)', Data)
                Lcd1602date = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) date', Data)
                Lcd1602time = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) time', Data)
                Lcd1602ram = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) ram', Data)
                Lcd1602wifi = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) wifi ([\S]*)', Data)
                Lcd1602 = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) ([\S ]*)', Data)

            if Addons["lcd1602g"]:
                Lcd1602clsg = re.findall(
                    r'LCD1602 cls', Data)
                Lcd1602rssg = re.findall(
                    r'LCD1602 rss ([0-9]+)', Data)
                Lcd1602blg = re.findall(
                    r'LCD1602 bl (on|off)', Data)
                Lcd1602rsscg = re.findall(
                    r'LCD1602 rss (off)', Data)
                Lcd1602wipg = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) wip', Data)
                Lcd1602eipg = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) eip', Data)
                Lcd16022g = re.findall(
                    r'LCD1602 ([\S ]*)', Data)
                Lcd1602dateg = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) date', Data)
                Lcd1602timeg = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) time', Data)
                Lcd1602ramg = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) ram', Data)
                Lcd1602wifig = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) wifi ([\S]*)', Data)
                Lcd1602g = re.findall(
                    r'LCD1602 ([0-9]*),([0-9]*) ([\S ]*)', Data)

            if Addons["Lcd2004A"] or Addons["Lcd2004B"] or Addons["Lcd2004C"] or Addons["Lcd2004D"] or Addons["Lcd2004"]:
                Lcd2004cls = re.findall(
                    r'LCD2004([A-D]?) cls', Data)
                Lcd2004rss = re.findall(
                    r'LCD2004([A-D]?) rss ([0-9]+)', Data)
                Lcd2004bl = re.findall(
                    r'LCD2004([A-D]?) bl (on|off)', Data)
                Lcd2004rssc = re.findall(
                    r'LCD2004([A-D]?) rss (off)', Data)
                Lcd2004wip = re.findall(
                    r'LCD2004([A-D]?) ([0-9]*),([0-9]*) wip', Data)
                Lcd2004eip = re.findall(
                    r'LCD2004([A-D]?) ([0-9]*),([0-9]*) eip', Data)
                Lcd20042 = re.findall(
                    r'LCD2004([A-D]?) ([\S ]*)', Data)
                Lcd2004date = re.findall(
                    r'LCD2004([A-D]?) ([0-9]*),([0-9]*) date', Data)
                Lcd2004time = re.findall(
                    r'LCD2004([A-D]?) ([0-9]*),([0-9]*) time', Data)
                Lcd2004 = re.findall(
                    r'LCD2004([A-D]?) ([0-9]*),([0-9]*) ([\S ]*)', Data)
                Lcd2004ram = re.findall(
                    r'LCD2004([A-D]?) ([0-9]*),([0-9]*) ram', Data)
                Lcd2004wifi = re.findall(
                    r'LCD2004([A-D]?) ([0-9]*),([0-9]*) wifi ([\S]*)', Data)

            if Addons["lcd2004g"]:
                Lcd2004clsg = re.findall(
                    r'LCD2004 cls', Data)
                Lcd2004rssg = re.findall(
                    r'LCD2004 rss ([0-9]+)', Data)
                Lcd2004blg = re.findall(
                    r'LCD2004 bl (on|off)', Data)
                Lcd2004rsscg = re.findall(
                    r'LCD2004 rss (off)', Data)
                Lcd2004wipg = re.findall(
                    r'LCD2004 ([0-9]*),([0-9]*) wip', Data)
                Lcd2004eipg = re.findall(
                    r'LCD2004 ([0-9]*),([0-9]*) eip', Data)
                Lcd20042g = re.findall(
                    r'LCD2004 [\S ]*', Data)
                Lcd2004dateg = re.findall(
                    r'LCD2004 ([0-9]*),([0-9]*) date', Data)
                Lcd2004timeg = re.findall(
                    r'LCD2004 ([0-9]*),([0-9]*) time', Data)
                Lcd2004g = re.findall(
                    r'LCD2004 ([0-9]*),([0-9]*) ([\S ]*)', Data)
                Lcd2004ramg = re.findall(
                    r'LCD2004 ([0-9]*),([0-9]*) ram', Data)
                Lcd2004wifig = re.findall(
                    r'LCD2004 ([0-9]*),([0-9]*) wifi ([\S]*)', Data)

            Lcd8544int = Lcd1602int = Lcd1602intg = False
            Lcd2004int = Lcd2004intg = False

            Lcd8544int = re.findall(r'LCD8544 init', Data)
            Lcd1602int = re.findall(r'LCD1602 init', Data)
            Lcd1602intg = re.findall(r'LCD1602g init', Data)
            Lcd2004int = re.findall(r'LCD2004([A-D]?)[ ]?init', Data)
            Lcd2004intg = re.findall(r'LCD2004g init', Data)

            Ledcls = re.findall(r'LEDCLS', Data)
            SLedcls = re.findall(r'SLEDCLS', Data)
            SLedrss = re.findall(r'SLEDRSS(KILL|OFF)?', Data)
            Ledrss = re.findall(r'LEDRSS(KILL|OFF)?', Data)
            SLed = re.findall(r'SLED[\S ]*', Data)
            Led = re.findall(r'LED[\S ]*', Data)
            LedP = re.findall(r'LPRINT[\S ]*', Data)
            LedP2 = re.findall(r'SPRINT[\S ]*', Data)
            EEPROM = re.findall(r'(EEP 0x[0-9a-fA-F]+ 0x[0-9a-fA-F]+ [\S ]*)', Data)

            if SLedrss:
                SLed = ""
                print SLedrssPID, SLedrss
                if SLedrss[0] == "KILL" or SLedrss[0] == "OFF":
                    if SLedrssPID != False:
                        os.kill(SLedrssPID, signal.SIGINT)
                        SLedrssPID = False
                        print "Kill / Off command"
                        continue
                elif SLedrssPID == False and SLedrss[0] != "KILL" or SLedrss[0] != "OFF":
                    DNull = open('/dev/null', 'w')
                    Proc = Popen(['python', 'lib/Matrix/Led16x16.py'], stdout=DNull, stderr=DNull)
                    SLedrssPID = Proc.pid
                Ledrss = ""
                Led = ""
                SLed = ""

            if Ledrss:
                if Ledrss[0] == "KILL" or Ledrss[0] == "OFF":
                    if LedrssPID:
                        os.kill(LedrssPID, signal.SIGINT)
                        LedrssPID = False
                        continue
                elif LedrssPID == False and Ledrss[0] != "KILL" or Ledrss[0] != "OFF":
                    DNull = open('/dev/null', 'w')
                    Proc = Popen(['python', 'lib/Matrix/Led24x16.py'], stdout=DNull, stderr=DNull)
                    LedrssPID = Proc.pid
                Led = ""

            if Ledcls:
                Matrix(" "*8)
                Led = False
                SLedcls = False

            if SLedcls:
                SMatrix(""*6)
                Led = False
                SLed = False

            if Lcd8544int:
                Addons["lcd8544"] = True

            if Lcd1602int:
                try:
                    print("LCD1602 Initialising.")
                    lcd1602 = Adafruit_CharLCDPlate()
                    time.sleep(0.1)
                    lcd1602.begin(16, 2)
                    Addons["lcd1602"] = True
                    print("LCD1602 Initialised.")
                    lcd1602.backlight(lcd1602.RED)
                except:
                    Addons["lcd1602"] = False

            if Lcd1602intg:
                try:
                    lcd1602g = Adafruit_CharLCD()
                    time.sleep(0.1)
                    lcd1602g.begin(16, 2)
                    Addons["lcd1602g"] = True
                    GPIO.output(18, True)
                except:
                    Addons["lcd1602g"] = False

            if Lcd2004int:
                try:
                    In = Lcd2004int[0]
                    if len(In) == 0:
                        Char = ""
                    else:
                        Char = In[0]
                    if Char == "A":
                        Lcd2004A = Adafruit_CharLCDPlate(addr=0x24)
                        Lcd2004A.begin(20, 4)
                        Lcd2004A.backlight(Lcd2004A.RED)
                        Addons["Lcd2004A"] = True
                    elif Char == "B":
                        Lcd2004B = Adafruit_CharLCDPlate(addr=0x25)
                        Lcd2004B.begin(20, 4)
                        Lcd2004B.backlight(Lcd2004B.RED)
                        Addons["Lcd2004B"] = True
                    elif Char == "C":
                        Lcd2004C = Adafruit_CharLCDPlate(addr=0x26)
                        Lcd2004C.begin(20, 4)
                        Lcd2004C.backlight(Lcd2004C.RED)
                        Addons["Lcd2004C"] = True
                    elif Char == "D":
                        Lcd2004D = Adafruit_CharLCDPlate(addr=0x27)
                        Lcd2004D.begin(20, 4)
                        Lcd2004D.backlight(Lcd2004D.RED)
                        Addons["Lcd2004D"] = True
                    elif Char == "":
                        lcd2004 = Adafruit_CharLCDPlate(addr=0x21)
                        lcd2004.begin(20, 4)
                        lcd2004.backlight(lcd2004.RED)
                        Addons["Lcd2004"] = True
                except Exception:
                    print "Something went wrong with Initialising LCD2004."
                Lcd2004 = False
                Lcd20042 = False
            if Lcd2004intg:
                try:
                    lcd2004g = Adafruit_CharLCD_V2()
                    time.sleep(0.1)
                    lcd2004g.begin(20, 4)
                    Addons["lcd2004g"] = True
                except:
                    Addons["lcd2004g"] = False

            if Lcd1602rss:
                print "RSS launched"
                In = Lcd1602rss[0]
                Args = ["python", "lib/RSS_Feed_I2C_1602.py", In]
                if not RSSRunning:
                    RSSFeed = subprocess.Popen(Args)
                    RSS1602FeedPID = RSSFeed.pid
                    RSSRunning = True
                    print("Starting RSS Feed with PID:" + str(RSS1602FeedPID))
                else:
                    print("RSS Already running.")

            if Lcd1602rssc:
                if RSS1602FeedPID == 9999:
                    pass
                print("Killing Process: " + str(RSS1602FeedPID))
                if RSSRunning:
                    os.kill(RSS1602FeedPID, signal.SIGINT)
                    RSSRunning = False
                lcd1602.clear()
                RSS1602FeedPID = 9999

            if Lcd1602date:
                In = Lcd1602date[0]
                X = int(In[0])
                Y = int(In[1])
                if Day == 1:
                    Suf = "st"
                elif Day == 2:
                    Suf = "nd"
                elif Day == 3:
                    Suf = "rd"
                else:
                    Suf = "th"
                print Year, type(Year)
                Date = Date[0:2]
                Month = Month[0:3]
                Year = Year[2:4]
                Day = Day + Suf
                Data = "{0} {1} {2} {3}".format(Date, Day, Month, Year)
                lcd1602.setCursor(X, Y)
                lcd1602.message(Data)
                Lcd1602 = False
                Lcd16022 = False

            if Lcd1602time:
                In = Lcd1602time[0]
                X = int(In[0])
                Y = int(In[1])
                Hour = time.strftime("%H")
                Minute = time.strftime("%M")
                Second = time.strftime("%S")
                Second = Second[:]
                print Second
                Data = ("%s:%s.%s" % (Hour, Minute, Second))
                lcd1602.setCursor(X, Y)
                lcd1602.message(Data)
                Lcd1602 = False
                Lcd16022 = False

            if Lcd1602cls:
                lcd1602.clear()
                Lcd1602 = False
                Lcd16022 = False
            if Lcd1602bl:
                print Lcd1602bl[0]
                Bl = Lcd1602bl[0]
                if Bl == "on":
                    lcd1602.backlight(lcd1602.RED)
                elif Bl == "off":
                    lcd1602.backlight(lcd1602.BLUE)
                Lcd1602 = False
                Lcd16022 = False
            if Lcd1602wip:
                In = Lcd1602wip[0]
                X = int(In[0])
                Y = int(In[1])
                cmd = [
                    "ip addr show",
                    "wlan0 |",
                    "grep inet |",
                    "awk '{print $2}' |",
                    "cut -d/ -f1"
                ]
                Run = subprocess.call(cmd, shell=True, stdout=PIPE)
                IP = Run.communicate()[0]
                lcd1602.setCursor(X, Y)
                lcd1602.message(IP)
                Lcd1602 = False
                Lcd16022 = False

            if Lcd1602eip:
                In = Lcd1602eip[0]
                X = int(In[0])
                Y = int(In[1])
                cmd = [
                    "ip addr show",
                    "wlan0 |",
                    "grep inet |",
                    "awk '{print $2}' |",
                    "cut -d/ -f1"
                ]
                Run = subprocess.call(cmd, shell=True, stdout=PIPE)
                IP = Run.communicate()[0]
                lcd1602.setCursor(X, Y)
                lcd1602.message(IP)
                Lcd1602 = False
                Lcd16022 = False
                
            if Lcd1602ram:
                In = Lcd1602ram[0]
                X = int(In[0])
                Y = int(In[1])
                Total, Used, Usage = GetRam()
                Total = int(float(Total))
                Used = int(float(Used))
                Data = "{0}/{1} ".format(Used, Total) + Usage + "%"
                lcd1602.setCursor(X, Y)
                lcd1602.message(Data)
                Lcd1602 = False
                Lcd16022 = False

            if Lcd1602wifi:
                In = Lcd1602wifi[0]
                X = int(In[0])
                Y = int(In[1])
                Cmd = In[2]
                SSID, Signal, Link = GetWifi()
                if Cmd == "ssid":
                    if SSID[:2] == "No":
                        Data = "No WiFi."
                    else:
                        Data = SSID
                elif Cmd == "signal":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    Data = Link
                elif Cmd == "ip":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    else:
                        cmd = "ip addr show wlan0 | grep inet | awk '{print $2}' | cut -d/ -f1"
                        Run = Popen(cmd, stdout=PIPE, shell=True)
                        IP = Run.communicate()[0]
                        Data = IP[:-1]
                elif Cmd == "gsig":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    else:
                        Sig = int(Link[:2])
                        A = (Sig / 70) * 100
                        A = int(A / 25)
                        Data = Link + " [" + "="*A + " "*(4-A) + "]"
                lcd1602.setCursor(X, Y)
                lcd1602.message(Data)
                Lcd1602 = False
                Lcd16022 = False

            if Lcd1602:
                Lcd1602 = Lcd1602[0]
                X = Lcd1602[0]
                Y = Lcd1602[1]
                Message = Lcd1602[2]
                lcd1602.setCursor(int(X), int(Y))
                lcd1602.message(Message)
                Lcd16022 = False

            if Lcd16022:
                Message = Lcd16022[0]
                Message = Message.replace('LCD1602 ', '')
                lcd1602.setCursor(0, 0)
                lcd1602.message(Message)

            if Lcd1602rssg:
                In = Lcd1602rssg[0]
                Args = ["python", "lib/Scroll_RSS_GP_1602.py", In]
                if not RSSRunning and In != '':
                    RSSFeed = Popen(Args)
                    RSS1602FeedPID = RSSFeed.pid
                    RSSRunning = True
                    print("Starting RSS Feed with PID:" + str(RSS1602FeedPID))

            if Lcd1602rsscg:
                In = Lcd1602rsscg[0]
                if In == "off" and RSSRunning:
                    print("Killing Process: " + str(RSS1602FeedPID))
                    try:
                        os.kill(RSS1602FeedPID, signal.SIGINT)
                    except:
                        print("Can't kill a dead Process")
                    print "Killed Process"
                    RSSRunning = False
                    RSS1602FeedPID = 9999

            if Lcd1602clsg:
                lcd1602g.clear()
                Lcd1602g = False
                Lcd16022g = False

            if Lcd1602dateg:
                In = Lcd1602dateg[0]
                X = int(In[0])
                Y = int(In[1])
                Date = time.strftime("%A")
                Day = time.strftime("%d")
                Month = time.strftime("%B")
                Year = time.strftime("%Y")
                if Day == 1:
                    Suf = "st"
                elif Day == 2:
                    Suf = "nd"
                elif Day == 3:
                    Suf = "rd"
                else:
                    Suf = "th"
                print Year, type(Year)
                Date = Date[0:2]
                Month = Month[0:3]
                Year = Year[2:4]
                Day = Day + Suf
                Data = "{0} {1} {2} {3}".format(Date, Day, Month, Year)
                lcd1602g.setCursor(X, Y)
                lcd1602g.message(Data)
                Lcd1602g = False
                Lcd16022g = False

            if Lcd1602timeg:
                In = Lcd1602timeg[0]
                X = int(In[0])
                Y = int(In[1])
                Hour = time.strftime("%H")
                Minute = time.strftime("%M")
                Second = time.strftime("%S")
                Second = Second[:]
                print Second
                Data = ("%s:%s.%s" % (Hour, Minute, Second))
                lcd1602g.setCursor(X, Y)
                lcd1602g.message(Data)
                Lcd1602g = False
                Lcd16022g = False

            if Lcd1602blg:
                Bl = Lcd1602blg[0]
                if Bl == "on":
                    GPIO.output(18, True)
                elif Bl == "off":
                    GPIO.output(18, False)
                Lcd1602g = False
                Lcd16022g = False

            if Lcd1602wipg:
                In = Lcd1602wipg[0]
                X = int(In[0])
                Y = int(In[1])
                cmd = [
                    "ip addr show",
                    "wlan0 |",
                    "grep inet |",
                    "awk '{print $2}' |",
                    "cut -d/ -f1"
                ]
                Run = Popen(cmd, stdout=PIPE)
                IP = Run.communicate()[0]
                lcd1602g.setCursor(X, Y)
                lcd1602g.message(IP)
                Lcd1602g = False
                Lcd16022g = False

            if Lcd1602eipg:
                In = Lcd1602eipg[0]
                X = int(In[0])
                Y = int(In[1])
                cmd = [
                    "ip addr show",
                    "eth0 |",
                    "grep inet |",
                    "awk '{print $2}' |",
                    "cut -d/ -f1"
                ]
                Run = Popen(cmd, stdout=PIPE)
                IP = Run.communicate()[0]
                lcd1602g.setCursor(X, Y)
                lcd1602g.message(IP)
                Lcd1602g = False
                Lcd16022g = False
                
            if Lcd1602ramg:
                In = Lcd1602ramg[0]
                X = int(In[0])
                Y = int(In[1])
                Total, Used, Usage = GetRam()
                Total = int(float(Total))
                Used = int(float(Used))
                Data = "{0}/{1} ".format(Used, Total) + Usage + "%"
                lcd1602g.setCursor(X, Y)
                lcd1602g.message(Data)
                Lcd1602g = False
                Lcd16022g = False

            if Lcd1602wifig:
                In = Lcd1602wifig[0]
                X = int(In[0])
                Y = int(In[1])
                Cmd = In[2]
                SSID, Signal, Link = GetWifi()
                if Cmd == "ssid":
                    if SSID[:2] == "No":
                        Data = "No WiFi."
                    else:
                        Data = SSID
                elif Cmd == "signal":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    Data = Link
                elif Cmd == "ip":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    else:
                        cmd = "ip addr show wlan0 | grep inet | awk '{print $2}' | cut -d/ -f1"
                        Run = Popen(cmd, stdout=PIPE, shell=True)
                        IP = Run.communicate()[0]
                        Data = IP[:-1]
                elif Cmd == "gsig":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    else:
                        Sig = int(Link[:2])
                        A = (Sig / 70) * 100
                        A = int(A / 25)
                        Data = Link + " [" + "="*A + " "*(4-A) + "]"
                lcd1602g.setCursor(X, Y)
                lcd1602g.message(Data)
                Lcd1602g = False
                Lcd16022g = False


            if Lcd1602g:
                Lcd1602 = Lcd1602g[0]
                X = Lcd1602[0]
                Y = Lcd1602[1]
                Message = Lcd1602[2]
                lcd1602g.setCursor(int(X), int(Y))
                lcd1602g.message(Message)
                Lcd16022g = False

            if Lcd16022g:
                Message = Lcd16022g[0]
                Message = Message.replace('LCD1602 ', '')
                lcd1602g.setCursor(0, 0)
                lcd1602g.message(Message)

            if Lcd2004rss:
                In = Lcd2004rss[0]
                Char = In[0]
                Speed = In[1]
                if Char == "A":
                    Addr = "24"
                elif Char == "B":
                    Addr = "25"
                elif Char == "C":
                    Addr = "26"
                elif Char == "D":
                    Addr = "27"
                elif Char == "":
                    Addr = "21"
                Args = ["python", "lib/RSS_Feed_I2C.py", Speed, Addr]
                if not RSSRunning and Char == "" and Addons["Lcd2004"]:
                    try:
                        RSSFeed = Popen(Args)
                        RSS2004FeedPID = RSSFeed.pid
                        RSSRunning = True
                        print("Starting RSS with PID:" + str(RSS2004FeedPID))
                    except:
                        print("Something went wrong with LCD2004rss")

                if not RSSRunningA and Char == "A" and Addons["Lcd2004A"]:
                    try:
                        RSSFeedA = Popen(Args)
                        RSS2004AFeedPID = RSSFeedA.pid
                        RSSRunningA = True
                        print("Starting RSS with PID:" + str(RSS2004AFeedPID))
                    except:
                        print("Something went wrong with LCD2004Arss")

                if not RSSRunningB and Char == "B" and Addons["Lcd2004B"]:
                    try:
                        RSSFeedB = Popen(Args)
                        RSS2004BFeedPID = RSSFeedB.pid
                        RSSRunningB = True
                        print("Starting RSS with PID:" + str(RSS2004BFeedPID))
                    except:
                        print("Something went wrong with LCD2004Brss")

                if not RSSRunningC and Char == "C" and Addons["Lcd2004C"]:
                    try:
                        RSSFeedC = Popen(Args)
                        RSS2004CFeedPID = RSSFeedC.pid
                        RSSRunningC = True
                        print("Starting RSS with PID:" + str(RSS2004CFeedPID))
                    except:
                        print("Something went wrong with LCD2004Crss")

                if not RSSRunningD and Char == "D" and Addons["Lcd2004D"]:
                    try:
                        RSSFeedD = Popen(Args)
                        RSS2004DFeedPID = RSSFeedD.pid
                        RSSRunningD = True
                        print("Starting RSS with PID:" + str(RSS2004DFeedPID))
                    except:
                        print("Something went wrong with LCD2004Drss")
                Lcd2004 = False
                Lcd20042 = False

            if Lcd2004rssc:
                In = Lcd2004rssc[0]
                Char = In[0]
                if Char == "A" and RSSRunningA:
                    print "Killing Process: {0}".format(RSS2004AFeedPID)
                    RSSRunningA = False
                    RSSFeedA.kill()
                    Lcd2004A.clear()
                    RSS2004AFeedPID = False
                elif Char == "B" and RSSRunningB:
                    print "Killing Process: {0}".format(RSS2004BFeedPID)
                    RSSRunningB = False
                    RSSFeedB.kill()
                    Lcd2004B.clear()
                    RSS2004BFeedPID = False
                elif Char == "C" and RSSRunningC:
                    print "Killing Process: {0}".format(RSS2004CFeedPID)
                    RSSRunningC = False
                    RSSFeedC.kill()
                    Lcd2004C.clear()
                    RSS2004CFeedPID = False
                elif Char == "D" and RSSRunningD:
                    print "Killing Process: {0}".format(RSS2004DFeedPID)
                    RSSRunningD = False
                    RSSFeedD.kill()
                    Lcd2004D.clear()
                    RSS2004DFeedPID = False
                elif Char == "" and RSSRunning:
                    print "Killing Process: {0}".format(RSS2004FeedPID)
                    RSSRunning = False
                    RSSFeed.kill()
                    lcd2004.clear()
                Lcd2004 = False
                Lcd20042 = False

            if Lcd2004date:
                In = Lcd2004date[0]
                Char = In[0]
                X = int(In[1])
                Y = int(In[2])
                Date = time.strftime("%A")
                Day = time.strftime("%d")
                Month = time.strftime("%B")
                Year = time.strftime("%Y")
                if Day == 1 or Day == 21 or Day == 31:
                    Suf = "st"
                elif Day == 2 or Day == 22:
                    Suf = "nd"
                elif Day == 3 or Day == 23:
                    Suf = "rd"
                else:
                    Suf = "th"
                print Year, type(Year)
                Date = Date[0:2]
                Month = Month[0:3]
                Year = Year[2:4]
                Day = Day + Suf
                Data = "{0} {1} {2} {3}".format(Date, Day, Month, Year)
                if Char == "A" and Addons["Lcd2004A"]:
                    Lcd2004A.setCursor(X, Y)
                    Lcd2004A.message(Data)
                elif Char == "B" and Addons["Lcd2004B"]:
                    Lcd2004B.setCursor(X, Y)
                    Lcd2004B.message(Data)
                elif Char == "C" and Addons["Lcd2004C"]:
                    Lcd2004C.setCursor(X, Y)
                    Lcd2004C.message(Data)
                elif Char == "D" and Addons["Lcd2004D"]:
                    Lcd2004D.setCursor(X, Y)
                    Lcd2004D.message(Data)
                elif Char == "" and Addons["Lcd2004"]:
                    lcd2004.setCursor(X, Y)
                    lcd2004.message(Data)
                Lcd2004 = False
                Lcd20042 = False

            if Lcd2004ram:
                In = Lcd2004ram[0]
                Char = In[0]
                X = int(In[1])
                Y = int(In[2])
                Total, Used, Usage = GetRam()
                #Data = "Used/Total: %s/%s, Usage: %s" % (Used, Total, Usage)
                Total = int(float(Total))
                Used = int(float(Used))
                Data = "{0}/{1} ".format(Used, Total) + Usage + "%"
                if Char == "A" and Addons["Lcd2004A"]:
                    Lcd2004A.setCursor(X, Y)
                    Lcd2004A.message(Data)
                elif Char == "B" and Addons["Lcd2004B"]:
                    Lcd2004B.setCursor(X, Y)
                    Lcd2004B.message(Data)
                elif Char == "C" and Addons["Lcd2004C"]:
                    Lcd2004C.setCursor(X, Y)
                    Lcd2004C.message(Data)
                elif Char == "D" and Addons["Lcd2004D"]:
                    Lcd2004D.setCursor(X, Y)
                    Lcd2004D.message(Data)
                elif Char == "" and Addons["Lcd2004"]:
                    lcd2004.setCursor(X, Y)
                    lcd2004.message(Data)
                Lcd2004 = False
                Lcd20042 = False

            if Lcd2004wifi:
                In = Lcd2004wifi[0]
                Char = In[0]
                X = int(In[1])
                Y = int(In[2])
                Cmd = In[3]
                SSID, Signal, Link = GetWifi()
                if Cmd == "ssid":
                    if SSID[:2] == "No":
                        Data = "No WiFi."
                    else:
                        Data = SSID
                elif Cmd == "signal":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    Data = Link
                elif Cmd == "ip":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    else:
                        cmd = "ip addr show wlan0 | grep inet | awk '{print $2}' | cut -d/ -f1"
                        Run = Popen(cmd, stdout=PIPE, shell=True)
                        IP = Run.communicate()[0]
                        Data = IP[:-1]
                elif Cmd == "gsig":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    else:
                        Sig = int(Link[:2])
                        A = (Sig / 70) * 100
                        A = int(A / 25)
                        Data = Link + " [" + "="*A + " "*(4-A) + "]"
                if Char == "A" and Addons["Lcd2004A"]:
                    Lcd2004A.setCursor(X, Y)
                    Lcd2004A.message(Data)
                elif Char == "B" and Addons["Lcd2004B"]:
                    Lcd2004B.setCursor(X, Y)
                    Lcd2004B.message(Data)
                elif Char == "C" and Addons["Lcd2004C"]:
                    Lcd2004C.setCursor(X, Y)
                    Lcd2004C.message(Data)
                elif Char == "D" and Addons["Lcd2004D"]:
                    Lcd2004D.setCursor(X, Y)
                    Lcd2004D.message(Data)
                elif Char == "" and Addons["Lcd2004"]:
                    lcd2004.setCursor(X, Y)
                    lcd2004.message(Data)
                Lcd2004 = False
                Lcd20042 = False

            if Lcd2004time:
                In = Lcd2004time[0]
                Char = In[0]
                X = int(In[1])
                Y = int(In[2])
                Hour = time.strftime("%H")
                Minute = time.strftime("%M")
                Second = time.strftime("%S")
                Second = Second[:]
                print Second
                Data = "{0}:{1}.{2}".format(Hour, Minute, Second)
                if Char == "A" and Addons["Lcd2004A"]:
                    Lcd2004A.setCursor(X, Y)
                    Lcd2004A.message(Data)
                elif Char == "B" and Addons["Lcd2004B"]:
                    Lcd2004B.setCursor(X, Y)
                    Lcd2004B.message(Data)
                elif Char == "C" and Addons["Lcd2004C"]:
                    Lcd2004C.setCursor(X, Y)
                    Lcd2004C.message(Data)
                elif Char == "D" and Addons["Lcd2004D"]:
                    Lcd2004D.setCursor(X, Y)
                    Lcd2004D.message(Data)
                elif Char == "" and Addons["Lcd2004"]:
                    lcd2004.setCursor(X, Y)
                    lcd2004.message(Data)
                Lcd2004 = False
                Lcd20042 = False
            if Lcd2004cls:
                Char = Lcd2004cls[0]
                if Char == "A" and Addons["Lcd2004A"]:
                    Lcd2004A.clear()
                elif Char == "B" and Addons["Lcd2004B"]:
                    Lcd2004B.clear()
                elif Char == "C" and Addons["Lcd2004C"]:
                    Lcd2004C.clear()
                elif Char == "D" and Addons["Lcd2004D"]:
                    Lcd2004D.clear()
                elif Char == "" and Addons["Lcd2004"]:
                    lcd2004.clear()
                Lcd2004 = False
                Lcd20042 = False
            if Lcd2004bl:
                In = Lcd2004bl[0]
                Char = In[0]
                Bl = In[1]
                if Char == "A" and Addons["Lcd2004A"]:
                    if Bl == "on":
                        Lcd2004A.backlight(Lcd2004A.RED)
                    elif Bl == "off":
                        Lcd2004A.backlight(Lcd2004A.BLUE)
                elif Char == "B" and Addons["Lcd2004B"]:
                    if Bl == "on":
                        Lcd2004B.backlight(Lcd2004B.RED)
                    elif Bl == "off":
                        Lcd2004B.backlight(Lcd2004B.BLUE)
                elif Char == "C" and Addons["Lcd2004C"]:
                    if Bl == "on":
                        Lcd2004C.backlight(Lcd2004C.RED)
                    elif Bl == "off":
                        Lcd2004C.backlight(Lcd2004C.BLUE)
                elif Char == "D" and Addons["Lcd2004D"]:
                    if Bl == "on":
                        Lcd2004D.backlight(Lcd2004D.RED)
                    elif Bl == "off":
                        Lcd2004D.backlight(Lcd2004D.BLUE)
                elif Char == "" and Addons["Lcd2004"]:
                    if Bl == "on":
                        lcd2004.backlight(lcd2004.RED)
                    elif Bl == "off":
                        lcd2004.backlight(lcd2004.BLUE)
                Lcd2004 = False
                Lcd20042 = False
            if Lcd2004wip:
                In = Lcd2004wip[0]
                Char = In[0]
                X = int(In[1])
                Y = int(In[2])
                cmd = "ip addr show wlan0 | grep inet | awk '{print $2}' | cut -d/ -f1"
                Run = Popen(cmd, stdout=PIPE, shell=True)
                IP = Run.communicate()[0]
                IP = IP[:-1]
                if Char == "A" and Addons["Lcd2004A"]:
                    Lcd2004A.setCursor(X, Y)
                    Lcd2004A.message(IP)
                elif Char == "B" and Addons["Lcd2004B"]:
                    Lcd2004B.setCursor(X, Y)
                    Lcd2004B.message(IP)
                elif Char == "C" and Addons["Lcd2004C"]:
                    Lcd2004C.setCursor(X, Y)
                    Lcd2004C.message(IP)
                elif Char == "D" and Addons["Lcd2004D"]:
                    Lcd2004D.setCursor(X, Y)
                    Lcd2004D.message(IP)
                elif Char == "" and Addons["Lcd2004"]:
                    lcd2004.setCursor(X, Y)
                    lcd2004.message(IP)
                Lcd2004 = False
                Lcd20042 = False
            if Lcd2004eip:
                In = Lcd2004eip[0]
                Char = In[0]
                X = int(In[1])
                Y = int(In[2])
                cmd = "ip addr show eth0 | grep inet | awk '{print $2}' | cut -d/ -f1"
                Run = Popen(cmd, stdout=PIPE, shell=True)
                IP = Run.communicate()[0]
                IP = IP[:-1]
                if Char == "A" and Addons["Lcd2004A"]:
                    Lcd2004A.setCursor(X, Y)
                    Lcd2004A.message(IP)
                elif Char == "B" and Addons["Lcd2004B"]:
                    Lcd2004B.setCursor(X, Y)
                    Lcd2004B.message(IP)
                elif Char == "C" and Addons["Lcd2004C"]:
                    Lcd2004C.setCursor(X, Y)
                    Lcd2004C.message(IP)
                elif Char == "D" and Addons["Lcd2004D"]:
                    Lcd2004D.setCursor(X, Y)
                    Lcd2004D.message(IP)
                elif Char == "" and Addons["Lcd2004"]:
                    lcd2004.setCursor(X, Y)
                    lcd2004.message(IP)
                Lcd2004 = False
                Lcd20042 = False

            if Lcd2004:
                In = Lcd2004[0]
                Char = In[0]
                X = int(In[1])
                Y = int(In[2])
                Message = In[3]
                Message = Message[:-1]
                if Char == "A" and Addons["Lcd2004A"]:
                    Lcd2004A.setCursor(X, Y)
                    Lcd2004A.message(Message)
                elif Char == "B" and Addons["Lcd2004B"]:
                    Lcd2004B.setCursor(X, Y)
                    Lcd2004B.message(Message)
                elif Char == "C" and Addons["Lcd2004C"]:
                    Lcd2004C.setCursor(X, Y)
                    Lcd2004C.message(Message)
                elif Char == "D" and Addons["Lcd2004D"]:
                    Lcd2004D.setCursor(X, Y)
                    Lcd2004D.message(Message)
                elif Char == "" and Addons["Lcd2004"]:
                    lcd2004.setCursor(X, Y)
                    lcd2004.message(Message)
                Lcd20042 = False
                
            if Lcd20042:
                In = Lcd20042[0]
                Char = In[0]
                Message = In[1]
                print "Message is :",Message
                if Char == "A" and Addons["Lcd2004A"]:
                    Lcd2004A.setCursor(0, 0)
                    Lcd2004A.message(Message)
                elif Char == "B" and Addons["Lcd2004B"]:
                    Lcd2004B.setCursor(0, 0)
                    Lcd2004B.message(Message)
                elif Char == "C" and Addons["Lcd2004C"]:
                    Lcd2004C.setCursor(0, 0)
                    Lcd2004C.message(Message)
                elif Char == "D" and Addons["Lcd2004D"]:
                    Lcd2004D.setCursor(0, 0)
                    Lcd2004D.message(Message)
                elif Char == "" and Addons["Lcd2004"]:
                    lcd2004.setCursor(0, 0)
                    lcd2004.message(Message)

            if Lcd2004rsscg:
                print("Killing Process: " + str(RSS2004FeedPID))
                RSSRunning = False
                try:
                    os.kill(RSS2004FeedPID, signal.SIGINT)
                except OSError:
                    print("Can't kill a dead Process.")
                Bl = "on"
                lcd2004g.clear()
                RSS2004FeedPID = 9999
                Lcd2004rssg = False
                Lcd2004g = False
                Lcd20042g = False
                
            if Lcd2004ramg:
                In = Lcd2004ramg[0]
                X = int(In[0])
                Y = int(In[1])
                Total, Used, Usage = GetRam()
                Total = int(float(Total))
                Used = int(float(Used))
                Data = "{0}/{1} {2}%".format(Used, Total, Usage)
                lcd2004g.setCursor(X, Y)
                lcd2004g.message(Data)
                Lcd2004g = False
                Lcd20042g = False

            if Lcd2004wifig:
                In = Lcd2004wifig[0]
                X = int(In[0])
                Y = int(In[1])
                Cmd = In[2]
                SSID, Signal, Link = GetWifi()
                if Cmd == "ssid":
                    if SSID[:2] == "No":
                        Data = "No WiFi."
                    else:
                        Data = SSID
                elif Cmd == "signal":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    Data = Link
                elif Cmd == "ip":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    else:
                        cmd = "ip addr show wlan0 | grep inet | awk '{print $2}' | cut -d/ -f1"
                        Run = Popen(cmd, stdout=PIPE, shell=True)
                        IP = Run.communicate()[0]
                        Data = IP[:-1]
                elif Cmd == "gsig":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    else:
                        Sig = int(Link[:2])
                        A = (Sig / 70) * 100
                        A = int(A / 25)
                        Data = Link + " [" + "="*A + " "*(4-A) + "]"
                lcd2004g.setCursor(X, Y)
                lcd2004g.message(Data)
                Lcd2004g = False
                Lcd20042g = False

            if Lcd2004rssg:
                Speed = Lcd2004rssg[0]
                Args = ["sudo", "python", "lib/Scroll_RSS_GP.py", Speed]
                if not RSSRunning:
                    RSSFeed = Popen(Args)
                    RSSRunning = True
                    RSS2004FeedPID = RSSFeed.pid
                    print("Starting RSS Feed with PID:" + str(RSS2004FeedPID))
                else:
                    print("RSS Already running.")
                Lcd2004g = False
                Lcd20042g = False

            if Lcd2004dateg:
                In = Lcd2004dateg[0]
                X = int(In[0])
                Y = int(In[1])
                Date = time.strftime("%A")
                Day = time.strftime("%d")
                Month = time.strftime("%B")
                Year = time.strftime("%Y")
                if Day == 1:
                    Suf = "st"
                elif Day == 2:
                    Suf = "nd"
                elif Day == 3:
                    Suf = "rd"
                else:
                    Suf = "th"
                Date = Date[0:2]
                Month = Month[0:3]
                Year = Year[2:4]
                Day = Day + Suf
                Data = "{0} {1} {2} {3}".format(Date, Day, Month, Year)
                lcd2004g.setCursor(X, Y)
                lcd2004g.message(Data)
                Lcd2004g = False
                Lcd20042g = False

            if Lcd2004timeg:
                In = Lcd2004timeg[0]
                X = int(In[0])
                Y = int(In[1])
                Hour = time.strftime("%H")
                Minute = time.strftime("%M")
                Second = time.strftime("%S")
                Second = Second[:]
                Data = ("%s:%s.%s" % (Hour, Minute, Second))
                lcd2004g.setCursor(X, Y)
                lcd2004g.message(Data)
                Lcd2004g = False
                Lcd20042g = False

            if Lcd2004clsg:
                lcd2004g.clear()
                Lcd2004g = False
                Lcd20042g = False
            if Lcd2004blg:
                Bl = Lcd2004blg[0]
                if Bl == "on":
                    GPIO.output(18, 1)
                elif Bl == "off":
                    GPIO.output(18, 0)
                Lcd2004g = False
                Lcd20042g = False
            if Lcd2004wipg:
                In = Lcd2004wipg[0]
                X = int(In[0])
                Y = int(In[1])
                cmd = [
                    "ip addr show",
                    "wlan0 |",
                    "grep inet |",
                    "awk '{print $2}' |",
                    "cut -d/ -f1"
                ]
                Run = Popen(cmd, stdout=PIPE)
                IP = Run.communicate()[0]
                IP = IP[:-1]
                lcd2004g.setCursor(X, Y)
                lcd2004g.message(IP)
                Lcd2004g = False
                Lcd20042g = False
            if Lcd2004eipg:
                In = Lcd2004eipg[0]
                X = int(In[0])
                Y = int(In[1])
                cmd = [
                    "ip addr show",
                    "eth0 |",
                    "grep inet |",
                    "awk '{print $2}' |",
                    "cut -d/ -f1"
                ]
                Run = Popen(cmd, stdout=PIPE)
                IP = Run.communicate()[0]
                IP = IP[:-1]
                lcd2004g.setCursor(X, Y)
                lcd2004g.message(IP)
                Lcd2004g = False
                Lcd20042g = False
            if Lcd2004g:
                In = Lcd2004g[0]
                X = int(In[0])
                Y = int(In[1])
                Message = In[2]
                Message = Message[:-1]
                lcd2004g.setCursor(X, Y)
                lcd2004g.message(Message)
                Lcd20042g = False

            if Lcd20042g:
                Message = Lcd20042g[0]
                Message = Message.replace('LCD2004 ', '')
                Message = Message[:-1]
                print Message
                lcd2004g.setCursor(0, 0)
                lcd2004g.message(Message)

            if Lcd8544rss:
                Speed = Lcd8544rss[0]
                Args = ["python", "lib/Scroll_RSS_8544.py", Speed]
                if not RSSRunning:
                    RSSFeed = Popen(Args)
                    RSSRunning = True
                    RSS8544FeedPID = RSSFeed.pid
                    print("Starting RSS Feed with PID:" + str(RSS8544FeedPID))
                    Lcd8544rssc = False
                else:
                    print("RSS Already running.")
                Lcd8544 = False
                Lcd85442 = False

            if Lcd8544rssc:
                Command = Lcd8544rssc[0]
                if RSS8544FeedPID == 9999:
                    continue
                print("Killing Process: " + str(RSS8544FeedPID))
                RSSRunning = False
                try:
                    os.kill(RSS8544FeedPID, signal.SIGINT)
                except:
                    print("")
                BacklLight = "on"
                lcd_cls8544()
                RSS8544FeedPID = 9999
                Lcd8544 = False
                Lcd85442 = False

            if Lcd8544bl:
                BackLight = Lcd8544bl[0]
                if BackLight == "on":
                    GPIO.output(18, True)
                elif BackLight == "off":
                    GPIO.output(18, False)
                Lcd8544 = False
                Lcd85442 = False
            if Lcd8544wip:
                Message = Lcd8544wip[0]
                X = Message[0]
                Y = Message[1]
                cmd = [
                    "ip addr show",
                    "wlan0 |",
                    "grep inet |",
                    "awk '{print $2}' |",
                    "cut -d/ -f1"
                ]
                p = Popen(cmd, stdout=PIPE)
                IP = p.communicate()[0]
                GoToXY8544(int(X), int(Y))
                lcd_text8544(IP)
                Lcd8544 = False
                Lcd85442 = False
            if Lcd8544eip:
                Message = Lcd8544eip[0]
                X = Message[0]
                Y = Message[1]
                cmd = [
                    "ip addr show",
                    "eth0 |",
                    "grep inet |",
                    "awk '{print $2}' |",
                    "cut -d/ -f1"
                ]
                p = Popen(cmd, stdout=PIPE)
                IP = p.communicate()[0]
                GoToXY8544(int(X), int(Y))
                lcd_text8544(IP)
                Lcd8544 = False
                Lcd85442 = False

            if Lcd8544date:
                In = Lcd8544date[0]
                print In
                X = int(In[0])
                Y = int(In[1])
                print X, Y
                Date = time.strftime("%A")
                Day = time.strftime("%d")
                Month = time.strftime("%B")
                Year = time.strftime("%Y")
                if Day == 1:
                    Suf = "st"
                elif Day == 2:
                    Suf = "nd"
                elif Day == 3:
                    Suf = "rd"
                else:
                    Suf = "th"
                print Year, type(Year)
                Date = Date[0:2]
                Month = Month[0:3]
                Year = Year[2:4]
                Day = Day + Suf
                Data = "{0} {1} {2} {3}".format(Date, Day, Month, Year)
                GoToXY8544(X, Y)
                lcd_text8544(Data)
                Lcd8544 = False
                Lcd85442 = False

            if Lcd8544time:
                In = Lcd8544time[0]
                print In
                X = int(In[0])
                Y = int(In[1])
                Hour = time.strftime("%H")
                Minute = time.strftime("%M")
                Second = time.strftime("%S")
                Second = Second[:]
                print Second
                Data = ("%s:%s.%s" % (Hour, Minute, Second))
                GoToXY8544(X, Y)
                lcd_text8544(Data)
                Lcd8544 = False
                Lcd85442 = False
                
            if Lcd8544ram:
                In = Lcd8544ram[0]
                X = int(In[0])
                Y = int(In[1])
                Total, Used, Usage = GetRam()
                Total = int(float(Total))
                Used = int(float(Used))
                Data = "{0}/{1} ".format(Used, Total) + Usage + "%"
                GoToXY8544(X, Y)
                lcd_text8544(Data)
                Lcd8544 = False
                Lcd85442 = False

            if Lcd8544wifi:
                In = Lcd8544wifi[0]
                X = int(In[0])
                Y = int(In[1])
                Cmd = In[2]
                SSID, Signal, Link = GetWifi()
                if Cmd == "ssid":
                    if SSID[:2] == "No":
                        Data = "No WiFi."
                    else:
                        Data = SSID
                elif Cmd == "signal":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    Data = Link
                elif Cmd == "ip":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    else:
                        cmd = "ip addr show wlan0 | grep inet | awk '{print $2}' | cut -d/ -f1"
                        Run = Popen(cmd, stdout=PIPE, shell=True)
                        IP = Run.communicate()[0]
                        Data = IP[:-1]
                elif Cmd == "gsig":
                    if Link[:2] == "No":
                        Data = "No WiFi."
                    else:
                        Sig = int(Link[:2])
                        A = (Sig / 70) * 100
                        A = int(A / 25)
                        Data = Link + " [" + "="*A + " "*(4-A) + "]"
                GoToXY8544(int(X), int(Y))
                lcd_text8544(Data)
                Lcd8544 = False
                Lcd85442 = False

            if Lcd8544c:
                Text = Lcd8544c[0]
                Contr = int(Text)
                Contr = Contr
                print hex(Contr)
                lcd_begin8544(Contr)
                Lcd8544 = False
                Lcd85442 = False
            if Lcd8544cls:
                lcd_cls8544()
                Lcd85442 = False
            if Lcd8544:
                In = Lcd8544[0]
                print In
                X = In[0]
                Y = In[1]
                Text = In[2]
                GoToXY8544(int(X), int(Y))
                lcd_text8544(Text)
                Lcd85442 = False
            if Lcd85442:
                X = "0"
                Y = "0"
                Text = Lcd85442[0]
                Text = Text.replace('LCD8544 ', '')
                GoToXY8544(int(X), int(Y))
                lcd_text8544(Text)
            if SLed:
                SLed = SLed[0]
                SLed = SLed.replace("update", "")
                SLed = SLed.replace("broadcast", "")
                SMatrix(SLed)
                Led = False

            if Led:
                Led[0] = Led[0].replace('update', '')
                Led[0] = Led[0].replace('broadcast', '')
                Matrix(Led)
            if LedP:
                LPrint(LedP)
            if LedP2:
                SPrint(LedP2)
            if EEPROM:
                EEPROM = EEPROM[0]
                Addr1 = EEPROM[4:9]
                Addr2 = EEPROM[8:14]
                Message = EEPROM[14:]
                Addr = Addr1 + Addr2
                print("EEPROM Message Received! 16Bit Address = " + Addr)
                print("Message: " + Message)
                Args = [
                    "sudo", "python", "lib/EEPROM/EEPROM-V2.py", Addr1,
                    Addr2, binascii.hexlify(Message)
                ]
                Popen(Args)

            Lcd8544cls = Lcd8544bl = Lcd8544wip = Lcd8544eip = False
            Lcd8544c = False
            Lcd8544date = Lcd8544time = Lcd85442 = Lcd8544 = False
            Lcd1602cls = Lcd1602bl = Lcd1602wip = Lcd1602eip = False
            Lcd1602c = False
            Lcd16022 = Lcd1602date = Lcd1602time = Lcd1602 = False
            Lcd1602clsg = Lcd1602blg = Lcd1602wipg = Lcd1602eipg = False
            Lcd1602cg = False
            Lcd16022g = Lcd1602dateg = Lcd1602timeg = Lcd1602g = False
            Lcd2004cls = Lcd2004bl = Lcd2004wip = Lcd2004eip = False
            Lcd2004c = False
            Lcd20042 = Lcd2004date = Lcd2004time = Lcd2004 = False
            Lcd2004clsg = Lcd2004blg = Lcd2004wipg = Lcd2004eipg = False
            Lcd2004cg = False
            Lcd20042g = Lcd2004dateg = Lcd2004timeg = Lcd2004g = False

            if Addons["lcd8544"]:
                if (
                        Lcd8544cls or Lcd8544bl or Lcd8544wip or
                        Lcd8544eip or Lcd8544c or Lcd8544date or
                        Lcd8544time or Lcd85442 or Lcd8544
                ):
                    Data = False
            if Addons["lcd1602"]:
                if (
                        Lcd1602cls or Lcd1602bl or Lcd1602wip or
                        Lcd1602eip or Lcd1602c or Lcd16022 or
                        Lcd1602date or Lcd1602time or Lcd1602
                ):
                    Data = False

            Data = Data.upper()
            Data = Data.split(' ')
            Length = len(Data)
            D1 = [(0, 0)]*(Length * 2)
            if not Data:
                Data = ["False"*2]
            for A in range(len(Data)):
                if Data[A] == "RTC":
                    Addons["RTC"] = True
                    os.system("clear")
                    List()
                if Data[A] == "UPDATE":
                    if Addons["MPU6050"]:
                        with open('/home/pi/MPUOUT.txt') as File:
                            Read = File.read()
                        Read = Read.split("\n")
                        #print Read
                        Last = Read[len(Read)-2]
                        if Last:
                            Last = Last.split(" ")
                            Last = filter(None, Last)
                            #print Last
                            X, Y, Z = Last[6], Last[7], Last[8]
                        else:
                            X=Y=Z=255
                        sendCmd( "sensor-update MPU-X {}".format( X ) )
                        sendCmd( "sensor-update MPU-Y {}".format( Y ) )
                        sendCmd( "sensor-update MPU-Z {}".format( Z ) )
                    for A in SPInputs:
                        Chip = int(A[0], 16)
                        Bank = A[1]
                        if Bank == 10:
                            Bank = 0x09
                            Bank2 = "A"
                        elif Bank == 26:
                            Bank = 0x19
                            Bank2 = "B"
                        Spisend(Chip, Bank, 0xFF)
                        Read = Spiread(Chip, Bank)
                        Spisend(Chip, Bank, 0x00)
                        Chip = (int(Chip) / 2) - 31
                        print Chip, Bank, '{:08b}'.format(Read)[::-1]
                        Read = '{:08b}'.format(Read)[::-1]
                        for A in range(len(Read)):
                            Out = Chip + Bank2 + "-" + str(A) + " "
                            RdA = Read[A]
                            sendCmd("sensor-update SPI" + Out + RdA)
                    for B in I2Inputs:
                        Addr = int(B[0], 16)
                        Reg = B[1]
                        print Addr, Reg
                        Bus.write_byte_data(Addr, Reg, 0xFF)
                        Read = Bus.read_byte_data(Addr, Reg)
                        Read = '{:08b}'.format(Read)
                        Read = Read[::-1]
                        print "Read: ", Read
                        for A in range(len(Read)):
                            Chip = Addr - 32
                            if Bank == 9:
                                Bank = "A"
                            elif Bank == 19:
                                Bank = "B"
                            TRead = Read[A:A+1]
                            Out = "{0}{1}-{2} {3}".format(Chip, Bank, A+1, TRead)
                            sendCmd("sensor-update I2C{0}".format(Out))

                    try:
                        for A in range(len(GpioS)):
                            if GpioS[A]:
                                Dir = GpioDir[A]
                                if Dir == "IN":
                                    Input = GPIO.input(A)
                                    print "Pin: {0} -> {1}".format(A, Input)
                                    sendCmd("sensor-update GPIO-{0} {1}".format(A, Input))
                    except:
                        print("Failed")

                    if Addons["TRACEIR"]:
                        for Pin in TraceList:
                            Pin = int(Pin)
                            State = GPIO.input(Pin)
                            sendCmd("sensor-update TraceIR-{0} {1}".format(Pin, State))

                    if Addons["MCP3424-1"]:
                        Ch = 0x6C
                        Ch2 = 0x6D
                        Adc = []

                        changechannel(Ch, 0x9C)
                        Adc.append(getadcreading(Ch, 0x9C))

                        changechannel(Ch, 0xBC)
                        Adc.append(getadcreading(Ch, 0xBC))

                        changechannel(Ch, 0xDC)
                        Adc.append(getadcreading(Ch, 0xDC))

                        changechannel(Ch, 0xFC)
                        Adc.append(getadcreading(Ch, 0xFC))

                        changechannel(Ch2, 0x9C)
                        Adc.append(getadcreading(Ch2, 0x9C))

                        changechannel(Ch2, 0xBC)
                        Adc.append(getadcreading(Ch2, 0xBC))

                        changechannel(Ch2, 0xDC)
                        Adc.append(getadcreading(Ch2, 0xDC))

                        changechannel(Ch2, 0xFC)
                        Adc.append(getadcreading(Ch2, 0xFC))

                        for A in range(len(Adc)):
                            Cha = "{0} {1}".format(A + 1, Adc[A])
                            sendCmd("sensor-update Adc1-Ch" + Cha)

                    if Addons["MCP3424-2"]:
                        Ch = 0x6E
                        Ch2 = 0x6F
                        Adc = []

                        changechannel(Ch, 0x9C)
                        Adc.append(getadcreading(Ch, 0x9C))

                        changechannel(Ch, 0xBC)
                        Adc.append(getadcreading(Ch, 0xBC))

                        changechannel(Ch, 0xDC)
                        Adc.append(getadcreading(Ch, 0xDC))

                        changechannel(Ch, 0xFC)
                        Adc.append(getadcreading(Ch, 0xFC))

                        changechannel(Ch2, 0x9C)
                        Adc.append(getadcreading(Ch2, 0x9C))

                        changechannel(Ch2, 0xBC)
                        Adc.append(getadcreading(Ch2, 0xBC))

                        changechannel(Ch2, 0xDC)
                        Adc.append(getadcreading(Ch2, 0xDC))

                        changechannel(Ch2, 0xFC)
                        Adc.append(getadcreading(Ch2, 0xFC))

                        for A in range(len(Adc)):
                            Chan = "{0} {1}".format(A + 1, Adc[A])
                            sendCmd("sensor-update Adc2-Ch" + Chan)

                    if Addons["MCP3002"]:
                        Mcp = MCP3002()
                        Out = Mcp.read_sensor(0)
                        sendCmd("sensor-update M3002-AD0 {0}".format(Out))
                        Out = Mcp.read_sensor(1)
                        sendCmd("sensor-update M3002-AD1 {0}".format(Out))

                    if Addons["lcd1602"]:
                        Btn = (
                            (lcd1602.SELECT, 'I2C1'),
                            (lcd1602.RIGHT, 'I2C2'),
                            (lcd1602.DOWN, 'I2C3'),
                            (lcd1602.UP, 'I2C4'),
                            (lcd1602.LEFT, 'I2C5'),
                            (lcd1602.six, 'I2C6'),
                            (lcd1602.seven, 'I2C7'),
                            (lcd1602.eight, 'I2C8')
                        )

                        for A in Btn:
                            if lcd1602.buttonPressed(A[0]):
                                sendCmd("sensor-update %s 1" % (A[1]))
                            else:
                                sendCmd("sensor-update %s 0" % (A[1]))

                    if Addons["Lcd2004"]:
                        Btn = (
                            (lcd2004.SELECT, 'I2C1'),
                            (lcd2004.RIGHT, 'I2C2'),
                            (lcd2004.DOWN, 'I2C3'),
                            (lcd2004.UP, 'I2C4'),
                            (lcd2004.LEFT, 'I2C5'),
                            (lcd2004.six, 'I2C6'),
                            (lcd2004.seven, 'I2C7'),
                            (lcd2004.eight, 'I2C8')
                        )

                        for A in Btn:
                            if lcd2004.buttonPressed(A[0]):
                                sendCmd("sensor-update Lcd2004-%s 1" % (A[1]))
                            else:
                                sendCmd("sensor-update Lcd2004-%s 0" % (A[1]))

                    if Addons["Lcd2004A"]:
                        Btn = (
                            (Lcd2004A.SELECT, 'I2C1'),
                            (Lcd2004A.RIGHT, 'I2C2'),
                            (Lcd2004A.DOWN, 'I2C3'),
                            (Lcd2004A.UP, 'I2C4'),
                            (Lcd2004A.LEFT, 'I2C5'),
                            (Lcd2004A.six, 'I2C6'),
                            (Lcd2004A.seven, 'I2C7'),
                            (Lcd2004A.eight, 'I2C8')
                        )

                        for A in Btn:
                            if Lcd2004A.buttonPressed(A[0]):
                                sendCmd("sensor-update Lcd2004A-%s 1" % (A[1]))
                            else:
                                sendCmd("sensor-update Lcd2004A-%s 0" % (A[1]))

                    if Addons["Lcd2004B"]:
                        Btn = (
                            (Lcd2004B.SELECT, 'I2C1'),
                            (Lcd2004B.RIGHT, 'I2C2'),
                            (Lcd2004B.DOWN, 'I2C3'),
                            (Lcd2004B.UP, 'I2C4'),
                            (Lcd2004B.LEFT, 'I2C5'),
                            (Lcd2004B.six, 'I2C6'),
                            (Lcd2004B.seven, 'I2C7'),
                            (Lcd2004B.eight, 'I2C8')
                        )

                        for A in Btn:
                            if Lcd2004B.buttonPressed(A[0]):
                                sendCmd("sensor-update Lcd2004B-%s 1" % (A[1]))
                            else:
                                sendCmd("sensor-update Lcd2004B-%s 0" % (A[1]))

                    if Addons["Lcd2004C"]:
                        Btn = (
                            (Lcd2004C.SELECT, 'I2C1'),
                            (Lcd2004C.RIGHT, 'I2C2'),
                            (Lcd2004C.DOWN, 'I2C3'),
                            (Lcd2004C.UP, 'I2C4'),
                            (Lcd2004C.LEFT, 'I2C5'),
                            (Lcd2004C.six, 'I2C6'),
                            (Lcd2004C.seven, 'I2C7'),
                            (Lcd2004C.eight, 'I2C8')
                        )

                        for A in Btn:
                            if Lcd2004C.buttonPressed(A[0]):
                                sendCmd("sensor-update Lcd2004C-%s 1" % (A[1]))
                            else:
                                sendCmd("sensor-update Lcd2004C-%s 0" % (A[1]))

                    if Addons["Lcd2004D"]:
                        Btn = (
                            (Lcd2004D.SELECT, 'I2C1'),
                            (Lcd2004D.RIGHT, 'I2C2'),
                            (Lcd2004D.DOWN, 'I2C3'),
                            (Lcd2004D.UP, 'I2C4'),
                            (Lcd2004D.LEFT, 'I2C5'),
                            (Lcd2004D.six, 'I2C6'),
                            (Lcd2004D.seven, 'I2C7'),
                            (Lcd2004D.eight, 'I2C8')
                        )

                        for A in Btn:
                            if Lcd2004D.buttonPressed(A[0]):
                                sendCmd("sensor-update Lcd2004D-%s 1" % (A[1]))
                            else:
                                sendCmd("sensor-update Lcd2004D-%s 0" % (A[1]))

                    if Addons["TMP102"]:
                        Tmp = Temp()
                        sendCmd("sensor-update Temp %.2f" % (Tmp))

                    if Addons["ADS1015"]:
                        Ads = Ads1x15()
                        Res1 = Ads.readADCSingleEnded(0)
                        Res2 = Ads.readADCSingleEnded(1)
                        Res3 = Ads.readADCSingleEnded(2)
                        Res4 = Ads.readADCSingleEnded(3)

                        Chan1 = ((Res1 * 3.0) / 1000.0)
                        Chan2 = ((Res2 * 3.0) / 1000.0)
                        Chan3 = ((Res3 * 3.0) / 1000.0)
                        Chan4 = ((Res4 * 3.0) / 1000.0)
                        print Chan1, Chan2, Chan3, Chan4

                        sendCmd("sensor-update ADC1 " + str(Chan1))
                        sendCmd("sensor-update ADC2 " + str(Chan2))
                        sendCmd("sensor-update ADC3 " + str(Chan3))
                        sendCmd("sensor-update ADC4 " + str(Chan4))

                    if Addons["DHT22"]:
                        DHTTemp, DHTHumid = GetDHT(DHTPin)
                        sendCmd("sensor-update DHT22-{0}Temp {1}".format(DHTPin, DHTTemp))
                        sendCmd("sensor-update DHT22-{0}Humidity {1}".format(DHTPin, DHTHumid))

                    if Addons["PIRSensor"]:
                        for A in PIRList:
                            In = GPIO.input(A)
                            sendCmd("sensor-update PIR-{0} {1}".format(A, In))

                    if Addons["BMP085"]:
                        try:
                            Bmp = BMP85(0x77, 3)
                            ETemp = Bmp.readTemperature()
                            EPressure = int(Bmp.readPressure())
                            EAlt = Bmp.readAltitude()
                            print ETemp, EPressure, EAlt
                            Pres = int(EPressure / 100.0)
                            sendCmd("sensor-update ETemp {0}".format(ETemp))
                            sendCmd("sensor-update EPressure {0}".format(Pres))
                            sendCmd("sensor-update EAlt {0}".format(EAlt))
                        except:
                            if Debug:
                                print("Something went wrong..")

                    if Addons["UltraSonic"]:
                        GPIO.output(GPIO_TRIGGER, False)
                        Sent += 1
                        ECHO = GPIO_ECHO
                        TRIG = GPIO_TRIGGER
                        print("Trigger / Echo > {0} / {1}".format(TRIG, ECHO))
                        Distance = Measure(ECHO, TRIG)
                        print "Distance: {0}".format(Distance)
                        if Distance == 0.1:
                            Failed += 1
                        else:
                            Finished += 1
                        sendCmd("sensor-update UltraSonic %.2f" % (Distance))
                    Hour = time.strftime("%H")
                    Minutes = time.strftime("%M")
                    Seconds = time.strftime("%S")
                    TTemp = Temp()
                    if TTemp:
                        sendCmd("sensor-update Temp {0}".format(TTemp))
                    if Addons["RTC"]:
                        sendCmd("sensor-update Day " + Day)
                        sendCmd("sensor-update Date " + Day2)
                        sendCmd("sensor-update Month " + Month)
                        sendCmd("sensor-update Year " + Year)
                        sendCmd("sensor-update Hour " + Hour)
                        sendCmd("sensor-update Minutes " + Minutes)
                        sendCmd("sensor-update Seconds " + Seconds)
            try:
                if Data[1] == "ULTRASONIC" and not ADDONS["UltraSonic"]:
                    Addons["UltraSonic"] = True
                    print("Ultra Sonic Device detected.")
            except:
                Addons["UltraSonic"] = False
            if Data[0] == "ENSOR-UPDATE":
                Data[0] = ""
            for A in range(len(Data)):
                TB18 = re.findall(r'TEMPB20', Data[A])
                LedInfo = re.findall(r'PINFO', Data[A])
                Echo = re.findall(r'(^E)(\d+)', Data[A])
                PinsOff = re.findall(r'PINSOFF', Data[A])
                ADC = re.findall(r'ADS1015INIT', Data[A])
                Trigger = re.findall(r'^T(\d+)', Data[A])
                MPU = re.findall(r'MPU6050INIT', Data[A])
                RFIDInit = re.findall('RFIDINIT([0-1])', Data[A])
                MCP4725i = re.findall(r'4725INIT', Data[A])
                MCP4802i = re.findall(r'4802INIT', Data[A])
                MCP3002i = re.findall(r'3002INIT', Data[A])
                Data2 = re.findall(r'G(\d+)(\D+)', Data[A])
                GLedPWMS = re.findall(r'GPWMLEDZ', Data[A])
                Spi = re.findall(r'SP(\d)(\D)(\d)', Data[A])
                BMP085i = re.findall(r'BMP085INIT', Data[A])
                LIRR = re.findall(r'LIRREMOTEINIT', Data[A])
                GPWM = re.findall(r'GSE(\d+)A(\d+)', Data[A])
                I2C20 = re.findall(r'I2C20[IN|INIT]', Data[A])
                SpiIn = re.findall(r'SP(\d)+(A|B)IN', Data[A])
                I2cIn = re.findall(r'I2(\d)+(A|B)IN', Data[A])
                SMotorAi = re.findall(r'^SMOTORAINIT', Data[A])
                SMotorBi = re.findall(r'^SMOTORBINIT', Data[A])
                PIRIn = re.findall(r'PIRINIT([0-9]+)', Data[A])
                AdcInit1 = re.findall(r'3424ADC1INIT', Data[A])
                AdcInit2 = re.findall(r'3424ADC2INIT', Data[A])
                I2C = re.findall(r'I2(\d+)(\D+)(\d+)', Data[A])
                DHT = re.findall(r'DHT22INIT([0-9]+)', Data[A])
                Bits = re.findall(r'BITS(\d)(\D)(\d+)', Data[A])
                Bit = re.findall(r'BIT(\d+)(\D+)(\d+)', Data[A])
                GPPWM = re.findall(r'G(\d+)PWM([-]?\d+)', Data[A])
                Servo = re.findall(r'^SE(\d+)(\D+)(\d+)', Data[A])
                LedPWMS = re.findall(r'PWMLED([0-9]+)STOP', Data[A])
                GTrace = re.findall(r'TRACEIRINIT([0-9]+)', Data[A])
                Bit8 = re.findall(r'8PIN(\d+)A(\d+|OFF|CLR)', Data[A])
                DMotor = re.findall(r'([A-Z])DM([-]*[0-9]*)', Data[A])
                Bits2 = re.findall(r'BITS(\d)(\D)(ON|OFF|CLR)', Data[A])
                Bit2 = re.findall(r'BIT(\d+)(\D+)(ON|OFF|CLR)', Data[A])
                MStep = re.findall(r'(\d+)MOTORDIS(\d+)S(\d+)', Data[A])
                DServo = re.findall(r'DSE([0-9]*)(\D+)([0-9]+)', Data[A])
                GLedPWM = re.findall(r'GPWMLED([A-C])S([0-9]+)', Data[A])
                LedPWM2 = re.findall(r'^PWMLED([0-9]+)B([0-9]+)', Data[A])
                MCP4725o = re.findall(r'4725OUT([-+]?\d+\.\d+|\d)', Data[A])
                SMotor = re.findall(r'SMOTOR([\d]+)([A|B])([-]*[\d]+)', Data[A])
                MCP4802o = re.findall(r'4802OUT([A|B])([-+]?\d+\.\d+|\d)', Data[A])
                I2SMotor = re.findall(r'I2SMOTOR([0-9]+)([A-B])([1-3])([-]*[0-9]+)', Data[A])
                I2MStep = re.findall(r'I2MOTORDIS(\d+)(\D)(\d)M(\d+)R(\d+[.]?[\d]?)', Data[A])
                LedPWM = re.findall(r'PWMLED([0-9]+)S([0-9]+)E([0-9]+)D([0-9]+)([TRUE|FALSE]*)', Data[A])

                if MPU:
                    CurDir = os.curdir
                    print CurDir
                    if MPU6050PID>0:
                        os.kill( MPU6050PID, signal.SIGINT )
                        MPU6050PID = 0
                    Proc = Popen(['lib/./demo_dmp'], stdout=open('/home/pi/MPUOUT.txt', 'w+'), stderr=PIPE)
                    Addons["MPU6050"] = True
                    MPU6050PID = Proc.pid
                if RFIDInit:
                    RFIDInit = RFIDInit[0]
                    Pin = RFIDInit[0]
                    DNull = open('/dev/null', 'w')
                    Args = ["python", "lib/RFID.py", Pin]
                    print "Starting RFID Reader"
                    if RFIDPid:
                        os.kill(RFIDPid, signal.SIGINT)
                        RFIDPid = ''
                    RFIDProc = Popen(Args, stdout=DNull, stderr=DNull)
                    RFIDPid = RFIDProc.pid
                    print "PID = ",RFIDPid

                if MStep:
                    MStep = MStep[0]
                    DNull = open('/dev/null', 'w')
                    Motor = MStep[0]
                    Encoder = MStep[1]
                    Step = MStep[2]
                    Args = ["sudo", "python", "lib/WheelEncoder.py", Encoder, Step, Motor]
                    Proc = Popen(Args, stdout=DNull, stderr=DNull)

                if I2MStep:
                    I2MStep = I2MStep[0]
                    DNull = open('/dev/null', 'w')
                    Addr = I2MStep[0]
                    Bank = I2MStep[1]
                    EPin = I2MStep[2]
                    MPin = I2MStep[3]
                    Rota = I2MStep[4]
                    Args = ["sudo", "python", "lib/I2Wheel.py", Addr, Bank, EPin, MPin, Rota]
                    Proc = Popen(Args, stdout=DNull, stderr=DNull)

                if LIRR:
                    DNull = open('/dev/null', 'w')
                    Args = ["sudo", "python", "lib/IRR.py"]
                    print "Starting LIRC Reader"
                    if LIRCPid:
                        os.kill(LIRCPid, signal.SIGINT)
                        LIRCDPid = ''
                    Proc = Popen(Args, stdout=DNull, stderr=DNull)
                    LIRCPid = Proc.pid
                    print "PID = ",LIRCPid

                if GLedPWMS:
                    os.kill(GLedPwmPID, signal.SIGINT)
                    GLedPwmRunning = False
                    GLedPwmPID = 9999

                if GLedPWM:
                    DNull = open('/dev/null', 'w')
                    In = GLedPWM[0]
                    Sect = In[0]
                    Speed = In[1]
                    if In == "Z" and GLedPwmRunning:
                        os.kill(GLedPwmPID, signal.SIGINT)
                        GLedPwmRunning = False
                        GLedPwmPID = 9999
                        continue
                    elif In == "Z" and not GLedPwmRunning:
                        continue
                    if GLedPwmRunning:
                        os.kill(GLedPwmPID, signal.SIGINT)
                        GLedPwmRunning = False
                        GLedPwmPID = 9999
                    Args = ["sudo", "python", "lib/GPwm.py", Sect, Speed]
                    Proc = Popen(Args, stdout=DNull, stderr=DNull)
                    GLedPwmRunning = True
                    GLedPwmPID = Proc.pid

                if LedPWM2:
                    DNull = open('/dev/null', 'w')
                    In = LedPWM2[0]
                    Addr = In[0]
                    Bright = In[1]
                    print "\nAddr: {}\nBrightness: {}\n".format(Addr, Bright)
                    Args = ["sudo", "python", "lib/Scroll.py", Addr, "64", "64", Bright]
                    if str(Addr) in LedPWM1:
                        if LedPWM1[str(Addr)] != "":
                            os.kill(LedPWM1[str(Addr)], signal.SIGINT)
                            LedPWM1[str(Addr)] = ""
                    Proc = Popen(Args, stdout=DNull, stderr=DNull)
                    LedPWM1[str(Addr)] = Proc.pid
                    
                if LedPWMS:
                    In = LedPWMS[0]
                    if str(In) in LedPWM1:
                        os.kill(LedPWM1[str(In)], signal.SIGINT)
                        LedPWM1[str(In)] = ""

                if LedPWM:
                    In = LedPWM[0]
                    Addr = In[0]
                    Start = In[1]
                    End = In[2]
                    Delay = In[3]
                    Tail = In[4]
                    if Tail == "TRUE":
                        Cmd = "--tail"
                        Tail = "True"
                    elif Tail == "FALSE":
                        Cmd = "--tail"
                        Tail = "False"
                    else:
                        Cmd = ""
                        Tail = ""
                    if Tail and Cmd:
                        Args = ["sudo", "python", "lib/Scroll.py", Addr, Start, End, Delay, Tail, Cmd]
                    else:
                        Args = ["sudo", "python", "lib/Scroll.py", Addr, Start, End, Delay]
                    print Args
                    print "\nAddr: {}\nStart: {}\nEnd: {}\nDelay: {}\n".format(Addr, Start, End, Delay)
                    DNull = open('/dev/null', 'w')
                    if str(Addr) in LedPWM1:
                        if LedPWM1[str(Addr)] != "":
                            os.kill(LedPWM1[str(Addr)], signal.SIGINT)
                            LedPWM1[str(Addr)] = ""
                    Proc = Popen(Args, stdout=DNull)
                    LedPWM1[str(Addr)] = Proc.pid

                if I2SMotor:
                    In = I2SMotor[0]
                    Addr = In[0]
                    Reg = In[1]
                    Side = In[2]
                    Time = In[3]
                    Null = open('/dev/null' , 'w')
                    if Reg == "A":
                        Reg = "0x12"
                    elif Reg == "B":
                        Reg = "0x13"
                    Args = ["sudo", "python", "lib/I2Stepper.py", Addr, Reg, Side, Time]
                    if Debug:
                        print "Arguments: ", Args
                    Popen(Args, stdout=Null, stderr=Null)
                    SMotor = ""

                if I2C20:
                    print "I2C20 Enabled."
                    Addr = "0x20"
                    Bank = 9
                    if (Addr, Bank) in I2Inputs:
                        print("Already in Inputs.")
                    else:
                        I2Inputs.append((Addr, Bank))

                if PIRIn:
                    In = PIRIn[0]
                    Addons["PIRSensor"] = True
                    if In not in PIRList:
                        PIRList.append(int(In))
                        GPIO.setup(int(In), 1)
                    else:
                        print "PIR Pin already enabled"

                if GTrace:
                    In = GTrace[0]
                    print In
                    Addons["TRACEIR"] = True
                    if In not in TraceList:
                        TraceList.append(int(In))
                        GPIO.setup(int(In), 1)
                    else:
                        print "Pin already enabled for Trace"

                if DHT:
                    In = DHT[0]
                    print "In = " + In
                    Addons["DHT22"] = True
                    DHTPin = In

                if PinsOff:
                    Pins = [
                        4, 7, 8, 9, 10, 11, 17, 18, 22,
                        23, 24, 25, 27, 28, 29, 30, 31
                    ]
                    for A in Pins:
                        try:
                            GPIO.output(A, 0)
                        except:
                            continue

                if GPPWM:
                    In = GPPWM[0]
                    Pin = In[0]
                    Speed = In[1]
                    if not ProcRunning:
                        Proc = Popen(["python","lib/PWM-RPIO.py"], stdin=PIPE, stderr=PIPE)
                        print "Process started."
                        PWMPID = Proc.pid
                        print "Process PID: " + str(PWMPID)
                        ProcRunning = True
                    Proc.stdin.write("{0},{1}\n".format(Pin, int(Speed)*19))
                if GPWM:
                    if not ProcRunning:
                        Proc = Popen(["python","lib/PWM-RPIO.py"], stdin=PIPE, stderr=PIPE)
                        print "Process started."
                        PWMPID = Proc.pid
                        print "Process PID: " + str(PWMPID)
                        ProcRunning = True
                    In = GPWM[0]
                    Pin = In[0]
                    Angle = int(In[1])
                    sendCmd("sensor-update GServo-{0} {1}".format(Pin, Angle))
                    Proc.stdin.write("{0},{1}\n".format(Pin, Angle))
                    print "Wrote {0} to pin {1}".format(Pin, Angle)

                if DMotor:
                    In = DMotor[0]
                    Pin = In[0]
                    Speed = In[1]
                    if Speed:
                        Speed = int(Speed)*19
                    if not ProcRunning:
                        Proc = Popen(["python","lib/PWM-RPIO.py"], stdin=PIPE, stderr=PIPE)
                        print "Process started."
                        PWMPID = Proc.pid
                        print "Process PID: " + str(PWMPID)
                        ProcRunning = True
                    try:
                        if Pin == "A":
                            if Speed > 0:
                                Proc.stdin.write("{0},{1}\n".format(17, Speed))
                                Proc.stdin.write("{0},{1}\n".format(18, 10))
                                print "+ " + str( Speed )
                            elif Speed < 0:
                                Proc.stdin.write("{0},{1}\n".format(17, 10))
                                Proc.stdin.write("{0},{1}\n".format(18, Speed * -1))
                                print "- " + str( int(Speed)*-1 )
                            else:
                                Proc.stdin.write("{0},{1}\n".format(17, 10))
                                Proc.stdin.write("{0},{1}\n".format(18, 10))
                        elif Pin == "B":
                            if Speed > 0:
                                Proc.stdin.write("{0},{1}\n".format(27, Speed))
                                Proc.stdin.write("{0},{1}\n".format(22, 10))
                                print "+ " + str( Speed )
                            elif Speed < 0:
                                Proc.stdin.write("{0},{1}\n".format(27, 10))
                                Proc.stdin.write("{0},{1}\n".format(22, Speed * -1))
                                print "- " + str( int(Speed)*-1 )
                            else:
                                Proc.stdin.write("{0},{1}\n".format(27, 10))
                                Proc.stdin.write("{0},{1}\n".format(22, 10))
                        elif Pin == "C":
                            if Speed > 0:
                                Proc.stdin.write("{0},{1}\n".format(23, Speed))
                                Proc.stdin.write("{0},{1}\n".format(24, 10))
                                print "+ " + str( Speed )
                            elif Speed < 0:
                                Proc.stdin.write("{0},{1}\n".format(23, 10))
                                Proc.stdin.write("{0},{1}\n".format(24, Speed * -1))
                                print "- " + str( int(Speed)*-1 )
                            else:
                                Proc.stdin.write("{0},{1}\n".format(23, 10))
                                Proc.stdin.write("{0},{1}\n".format(24, 10))      
                        elif Pin == "D":
                            if Speed > 0:
                                Proc.stdin.write("{0},{1}\n".format(25, Speed))
                                Proc.stdin.write("{0},{1}\n".format( 4, 10))
                                print "+ " + str( Speed )
                            elif Speed < 0:
                                Proc.stdin.write("{0},{1}\n".format(25, 10))
                                Proc.stdin.write("{0},{1}\n".format( 4, Speed * -1))
                                print "- " + str( int(Speed)*-1 )
                            else:
                                Proc.stdin.write("{0},{1}\n".format(25, 10))
                                Proc.stdin.write("{0},{1}\n".format(4, 10))
                        elif Pin == "E":
                            if Speed > 0:
                                Proc.stdin.write("{0},{1}\n".format(28, Speed))
                                Proc.stdin.write("{0},{1}\n".format(29, 10))
                                print "+ " + str( Speed )
                            elif Speed < 0:
                                Proc.stdin.write("{0},{1}\n".format(28, 10))
                                Proc.stdin.write("{0},{1}\n".format(29, Speed * -1))
                                print "- " + str( int(Speed)*-1 )
                            else:
                                Proc.stdin.write("{0},{1}\n".format(28, 10))
                                Proc.stdin.write("{0},{1}\n".format(29, 10)) 
                        elif Pin == "F":
                            if Speed > 0:
                                Proc.stdin.write("{0},{1}\n".format(30, Speed))
                                Proc.stdin.write("{0},{1}\n".format(31, 10))
                                print "+ " + str( Speed )
                            elif Speed < 0:
                                Proc.stdin.write("{0},{1}\n".format(30, 10))
                                Proc.stdin.write("{0},{1}\n".format(31, Speed * -1))
                                print "- " + str( int(Speed)*-1 )
                            else:
                                Proc.stdin.write("{0},{1}\n".format(30, 10))
                                Proc.stdin.write("{0},{1}\n".format(31, 10))
                        elif Pin == "R":
                            Proc.stdin.write("STOP\n")
                            ProcRunning = False
                        else:
                            print "Only A-F Supported. Speeds -100 to 100. [R]eset"
                    except IOError:
                        if Proc.poll() != 0:
                            Proc = Popen(["python","lib/PWM-RPIO.py"], stdin=PIPE, stderr=PIPE)
                            print "Process started."
                            PWMPID = Proc.pid
                            print "Process PID: " + str(PWMPID)
                            ProcRunning = True

                if ADC:
                    Addons["ADS1015"] = True

                if SMotorAi:
                    Addons["SMotorA"] = True
                    GPIO.setup(17, GPIO.OUT)
                    GPIO.setup(18, GPIO.OUT)
                    GPIO.setup(27, GPIO.OUT)
                    GPIO.setup(22, GPIO.OUT)

                if SMotorBi:
                    Addons["SMotorB"] = True
                    GPIO.setup(23, GPIO.OUT)
                    GPIO.setup(24, GPIO.OUT)
                    GPIO.setup(25, GPIO.OUT)
                    GPIO.setup(4, GPIO.OUT)

                if AdcInit1:
                    Addons["MCP3424-1"] = True

                if AdcInit2:
                    Addons["MCP3424-2"] = True

                if SMotor:
                    In = SMotor[0]
                    Speed = int(In[0])
                    Chan = In[1]
                    Ang = int(In[2])
                    Delay = Speed / 1000.0
                    if Chan == "A":
                        if Ang > 0:
                            Stepper_Forward_A(Delay, Ang)
                        elif Ang < 0:
                            Stepper_Reverse_A(Delay, Ang)
                    elif Chan == "B":
                        if Ang > 0:
                            Stepper_Forward_B(Delay, Ang)
                        elif Ang < 0:
                            Stepper_Reverse_B(Delay, Ang)

                if BMP085i and not Addons["BMP085"]:
                    print("BMP085 Initialised.")
                    Addons["BMP085"] = True
                if MCP4725i:
                    Addons["MCP4725"] = True
                    print("MCP4725 = True")
                    McpDAC = MCP4725()
                    print McpDAC

                if MCP4802i:
                    Addons["MCP4802"] = True
                    unload_spi = Popen(
                        'sudo rmmod spi_bcm2708',
                        shell=True, stdout=PIPE
                    )

                    start_spi = Popen(
                        'sudo modprobe spi_bcm2708',
                        shell=True, stdout=PIPE
                    )

                    time.sleep(3)
                    spi = spidev.SpiDev()
                    spi.open(0, 1)
                    print("MCP4802 = True")

                if MCP3002i:
                    print("MCP3002 Set to True.")
                    Addons["MCP3002"] = True

                if MCP4725o and Addons["MCP4725"]:
                    In = float(MCP4725o[0])
                    print In
                    if In < 3.3:
                        McpDAC.analogWriteVolt(0, float(In)+0.005)

                if MCP4802o and Addons["MCP4802"]:
                    In = MCP4802o[0]
                    Chan = In[0]
                    Volt = In[1]
                    print Chan, Volt
                    if Chan == "A":
                        SetV1(Volt)
                    elif Chan == "B":
                        SetV2(Volt)
                    else:
                        print("Something went wrong...")

                if SpiIn:
                    In = SpiIn[0]
                    Chip = In[0]
                    Bank = In[1]
                    Hex = hex((int(Chip) * 2) + 62)
                    if Bank == "A":
                        Bank = 0x0A
                    elif Bank == "B":
                        Bank = 0x1A
                    else:
                        Bank = 0x0A
                    if (Hex, Bank) not in SPInputs:
                        print("Added :", Hex, Bank)
                        SPInputs.append((Hex, Bank))

                if I2cIn:
                    In = I2cIn[0]
                    Chip = hex(int(In[0]) + 31)
                    Bank = In[1]
                    if Bank == "A":
                        Bank = 0x12
                    elif Bank == "B":
                        Bank = 0x13
                    print In, Chip, Bank
                    if (Chip, Bank) in I2Inputs:
                        print("Already in Inputs.")
                    else:
                        I2Inputs.append((Chip, Bank))

                if LedInfo:
                    if GPIO_ECHO != 0 and GPIO_TRIGGER != 0:
                        Tmp = str(Temp())
                        Sonic = str(Measure(GPIO_ECHO, GPIO_TRIGGER))
                        Text = ("LPRINT{0}{1}".format(Tmp[:4], Sonic[:4]))
                        LPrint(Text)

                if TB18:
                    TempB18 = read_B18()

                if Bits:
                    Bits = Bits[0]
                    Chip = int(Bits[0])
                    Bank = Bits[1]
                    Bin = Bits[2]
                    Addr = "0x" + str((Chip * 2) + 38)
                    if Chip == 6:
                        Addr = "0x4A"
                    elif Chip == 7:
                        Addr = "0x4C"
                    elif Chip == 8:
                        Addr = "0x4E"

                    if Bank.upper() == "A":
                        Bank = 0x0A
                    elif Bank.upper() == "B":
                        Bank = 0x1A
                    else:
                        Bank = 0x00
                    Bin = "0b" + Bin
                    print(Addr, Bank, Bin)
                    Spisend(int(Addr, 16), Bank, int(Bin, 2))
                if Bits2:
                    Bits2 = Bits2[0]
                    Chip = int(Bits2[0])
                    Bank = Bits2[1]
                    Cmd = Bits2[2]
                    print Chip, Bank, Cmd
                    Addr = "0x" + str((Chip*2) + 38)
                    if Chip == 6:
                        Addr = "0x4A"
                    elif Chip == 7:
                        Addr = "0x4C"
                    elif Chip == 8:
                        Addr = "0x4E"

                    if Bank.upper() == "A":
                        Bank = 0x0A
                    elif Bank.upper() == "B":
                        Bank = 0x1A
                    else:
                        Bank = 0x00

                    if Cmd.upper() == "OFF" or Cmd.upper() == "CLR":
                        Bin = 0x00
                    elif Cmd.upper() == "ON":
                        Bin = 0xFF

                    Spisend(int(Addr, 16), Bank, Bin)

                if Bit8:
                    Addr = Bit8[0][0]
                    Cmd = Bit8[0][1]
                    Bus = smbus.SMBus(1)
                    if not Bit8Setup:
                        Bus.write_byte_data(0x20, 0x00, 0x00)
                        Bus.write_byte_data(0x20, 0x01, 0x00)
                        Bit8Setup = True
                    Address = int("0x" + Addr, 16)
                    if Cmd == "1":
                        Bin = 0b00000001
                    elif Cmd == "2":
                        Bin = 0b00000010
                    elif Cmd == "3":
                        Bin = 0b00000100
                    elif Cmd == "4":
                        Bin = 0b00001000
                    elif Cmd == "5":
                        Bin = 0b00010000
                    elif Cmd == "6":
                        Bin = 0b00100000
                    elif Cmd == "7":
                        Bin = 0b01000000
                    elif Cmd == "8":
                        Bin = 0b10000000
                    elif Cmd == "OFF" or Cmd == "CLR":
                        Bin = 0b00000000
                        Cmd = "255"
                    else:
                        Bin = int(Cmd, 2)
                    print "Cmd = {0} Addr = {1} Bin = {2}".format(Cmd, Addr, Bin)
                    if Debug:
                        print(Bin, Addr, Cmd)
                    Read = Bus.read_byte_data(Address, 0x09)
                    if int(Cmd) < 8 and int(Cmd) > 0:
                        Bus.write_byte_data(Address, 0x09, Read ^ Bin)
                    else:
                        Bus.write_byte_data(Address, 0x09, Bin)
                if Bit2:
                    Addr = Bit2[0][0]
                    Bank = Bit2[0][1]
                    Cmd = Bit2[0][2]
                    Bus = smbus.SMBus(1)
                    Bit = False
                    if Cmd == "ON":
                        if Bank.upper() == "A":
                            Bus.write_byte_data(int("0x"+Addr, 16), 0x12, 0xFF)
                            Bus.write_byte_data(int("0x"+Addr, 16), 0x09, 0xFF)
                        elif Bank.upper() == "B":
                            Bus.write_byte_data(int("0x"+Addr, 16), 0x13, 0xFF)
                            Bus.write_byte_data(int("0x"+Addr, 16), 0x19, 0xFF)
                    if Cmd == "OFF" or Cmd == "CLR":
                        if Bank.upper() == "A":
                            Bus.write_byte_data(int("0x"+Addr, 16), 0x12, 0x00)
                            Bus.write_byte_data(int("0x"+Addr, 16), 0x09, 0x00)
                        elif Bank.upper() == "B":
                            Bus.write_byte_data(int("0x"+Addr, 16), 0x13, 0x00)
                            Bus.write_byte_data(int("0x"+Addr, 16), 0x19, 0x00)

                if DServo:
                    In = DServo[0]
                    Channel = int(In[0])
                    Bank = In[1].upper()
                    Angle = int(In[2])
                    if Bank == "A":
                        Addr = 0x41
                    elif Bank == "B":
                        Addr = 0x40

                    if Debug:
                        print("Chan: {0}\nAngle: {1}".format(Channel, Angle))
                    if Angle >= SMax:
                        Angle = SMax
                    elif Angle <= SMin:
                        Angle = SMin
                    SetAngle(Channel, Addr, Angle, "Set")
                    Servo = False

                if Servo:
                    Addr = 0x00
                    In = Servo[0]
                    Channel = int(In[0])
                    Bank = In[1]
                    AngleI = int(In[2])

                    if Bank[1:] == "-":
                        Mult = -1
                    else:
                        Mult = 1
                    if Bank[:1] == "B":
                        Addr = 0x40
                        Angle = SAngles2[Channel] + (int(AngleI)*Mult)
                        if Angle > SMax:
                            Angle = SMax
                        if Angle < SMin:
                            Angle = SMin
                        SAngles2[Channel] = Angle
                    else:
                        Addr = 0x41
                        Angle = SAngles[Channel] + (int(AngleI)*Mult)
                        if Angle >= SMax:
                            Angle = SMax
                        if Angle <= SMin:
                            Angle = SMin
                        SAngles[Channel] = Angle
                    SetAngle(Channel, Addr, Angle, "Add")
                if Bit:
                    Addr = Bit[0][0]
                    Bank = Bit[0][1]
                    Binary = Bit[0][2]
                    Binary = int(Binary, 2)
                    Bus = smbus.SMBus(1)
                    if Debug:
                        print Bit
                        print Bank, Binary
                    if Bank.upper() == "A":
                        Bus.write_byte_data(int("0x"+Addr, 16), 0x12, Binary)
                        Bus.write_byte_data(int("0x"+Addr, 16), 0x09, Binary)
                    elif Bank.upper() == "B":
                        Bus.write_byte_data(int("0x"+Addr, 16), 0x13, Binary)
                        Bus.write_byte_data(int("0x"+Addr, 16), 0x19, Binary)
                    else:
                        if Debug:
                            print("Incorrect Bank enter in Bit command.")
                if Trigger:
                    Trigger = int(Trigger[0])
                    GPIO_TRIGGER = Trigger
                    GPIO.setup(GPIO_TRIGGER, GPIO.OUT)
                    setup1 = True
                    continue
                if Echo:
                    Echo = int(Echo[0][1])
                    GPIO_ECHO = Echo
                    GPIO.setup(GPIO_ECHO, GPIO.OUT)
                    setup2 = True
                    continue
                ECHO = GPIO_ECHO
                TRIG = GPIO_TRIGGER
                if TRIG != 0 and ECHO != 0 and not Addons["UltraSonic"]:
                    Addons["UltraSonic"] = True
                    if Debug:
                        print("UltraSonic Set up!")
                if Spi:
                    Spi = Spi[0]
                    Bank = int(Spi[0])
                    Bank2 = int((Bank * 2) + 38)
                    Addr = int("0x" + str(Bank2), 16)

                    if Bank == 6:
                        Addr = 0x4A
                    elif Bank == 7:
                        Addr = 0x4C
                    elif Bank == 8:
                        Addr = 0x4E
                    print Spi, Addr
                    Reg = Spi[1].upper()
                    if Reg.upper() == "A":
                        Reg = 0x0A
                    elif Reg.upper() == "B":
                        Reg = 0x1A
                    else:
                        Reg = 0x00
                        print("Error. Only A or B. Try Again.")

                    Led = int(Spi[2])
                    Offset = 0b1 << (Led-1)
                    print Offset
                    Out = "0b"+'{:08b}'.format(Offset)
                    Read = Spiread(Addr, Reg)
                    Out = int(Out, 2) ^ Read
                    Spisend(Addr, Reg, Out)
                if Data2:
                    Data2 = Data2[0]
                    Pin = int(Data2[0])
                    Cmd = Data2[1].upper()
                    if Cmd == "OUT":
                        GpioS[Pin] = True
                        GpioDir[Pin] = "OUT"
                        GPIO.setup(Pin, GPIO.OUT)
                        print(str(Pin) + " set to OUT")
                    if Cmd == "IN":
                        GpioS[Pin] = True
                        GpioDir[Pin] = "IN"
                        GPIO.setup(Pin, GPIO.IN)
                        print(str(Pin) + " set to IN")
                    if Cmd == "ON" and GpioS[Pin]:
                        GPIO.output(Pin, 1)
                        print(str(Pin) + " set to ON")
                    if Cmd == "OFF" and GpioS[Pin]:
                        GPIO.output(Pin, 0)
                        print(str(Pin) + " set to OFF")
                if I2C:
                    if not I2CSetup:
                        I2CSetFunc()
                        I2CSetup = True

                    Addr = I2C[0][0]
                    Bank = I2C[0][1]
                    Led = I2C[0][2]
                    if Bank == "A":
                        Reg = 0x12
                    elif Bank == "B":
                        Reg = 0x13
                    if Led == "1":
                        Bin = 0b00000001
                    elif Led == "2":
                        Bin = 0b00000010
                    elif Led == "3":
                        Bin = 0b00000100
                    elif Led == "4":
                        Bin = 0b00001000
                    elif Led == "5":
                        Bin = 0b00010000
                    elif Led == "6":
                        Bin = 0b00100000
                    elif Led == "7":
                        Bin = 0b01000000
                    elif Led == "8":
                        Bin = 0b10000000
                    elif Led == "9":
                        Bin = 0b11111111
                    Bus = smbus.SMBus(1)
                    Read = Bus.read_byte_data(int("0x"+Addr, 16), Reg)
                    print Read
                    Bus.write_byte_data(int("0x"+Addr, 16), Reg,Read ^ Bin)

    except KeyboardInterrupt:
        Spisend(0x40, 0x0A, 0x00)
        Spisend(0x40, 0x1A, 0x00)
        Spisend(0x42, 0x0A, 0x00)
        Spisend(0x42, 0x1A, 0x00)
        Spisend(0x44, 0x0A, 0x00)
        Spisend(0x44, 0x1A, 0x00)
        Spisend(0x46, 0x0A, 0x00)
        Spisend(0x46, 0x1A, 0x00)
        Spisend(0x40, 0x05, 0x00)
        Spisend(0x42, 0x05, 0x00)
        Spisend(0x44, 0x05, 0x00)
        Spisend(0x46, 0x05, 0x00)
        GPIO.cleanup()
        os.system("clear")
        if Debug:
            print ("GPIO Cleaned up and exiting.")
        Sent = "Sent: {0}".format(Sent)
        Fin = " Finished: {0}".format(Finished)
        Failed = " Failed: {0}".format(Failed)
        if Failed == 0:
            print(Sent + Fin + Failed)
        sys.exit()
