#!/usr/bin/env python

#-------------------------------------------------------#
#                                                       #
#              Name: I2Wheel.py                         #
#         Author: James Clarke, Pridopia.               #
#       Website: http://www.pridopia.co.uk              #
#              Date: 11 / 03 / 14                       #
#                Version: 1.04                          #
#                                                       #
#-------------------------------------------------------#
#
#	Patch Notes 1.04:
#
#	Added function to accept floats ( 0.2 rotations with a minimum of 4 steps )
#
# Copyright 2013 Pridopia (www.pridopia.co.uk)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at:
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# /LICENSE.txt
#
# http://www.pridopia.com/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIC,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either-express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

import RPi.GPIO as GPIO
import smbus, time, sys

Address = int("0x"+str(sys.argv[1]), 16)
Bank = sys.argv[2].upper()
Bit = sys.argv[3]
Motor = int(sys.argv[4])
Rotate = float(sys.argv[5])

Holes = 20

tmp = Holes*Rotate

if tmp < 4:
	print "Minimum is 4 steps\n{} holes, Minimum step is {}".format(Holes, float(4.0/float(Holes)))
	Rotate = float(4.0/float(Holes))
	print Rotate

if Bank == "A":
	Bank = 0x12
elif Bank == "B":
	Bank = 0x13
else:
	Bank = 0x00

if Bit == "1":
	Byte = 0x01
elif Bit == "2":
	Byte = 0x02
elif Bit == "3":
	Byte = 0x04
elif Bit == "4":
	Byte = 0x08
elif Bit == "5":
	Byte = 0x10
elif Bit == "6":
	Byte = 0x20
elif Bit == "7":
	Byte = 0x40
elif Bit == "8":
	Byte = 0x80

Bus = smbus.SMBus(1)
GPIO.setmode(11)
GPIO.setup(Motor, GPIO.OUT)

def Write(addr, reg, data):
	Bus.write_byte_data(addr, reg, data)
	return

def Read(addr, reg):
	return Bus.read_byte_data(addr, reg)

def Detect():
	global Count
	global Running
	global Rotate
	Count = Count + 1
	sys.stdout.write("{} ".format(Count))
	sys.stdout.flush()
	if Count == int(Holes * Rotate)-3:
		Cnt = int(Holes*Rotate)
		GPIO.output(Motor, 0)
		sys.stdout.write("{} {} {}\n".format(Cnt-2, Cnt-1, Cnt))
		sys.stdout.flush()
		Running = False
	elif Count == Holes:
		Rotate = Rotate - 1
		Count = 0
		sys.stdout.write("\n")
		sys.stdout.flush()
	elif Count == 17 and Rotate == 0:
		sys.stdout.write("18 19 20\n")	# Fake writing out the next 2, Wheel will still spin after shutdown, which seems to be 2 / 20.
		print "Finished?"
		GPIO.output(Motor, 0)
		Running = False
Count = 0
Last = 0
Write(Address, 0x00, 0xFF)
Write(Address, 0x01, 0xFF)
Write(Address, Bank, 0x00)

Running = True
GPIO.output(Motor, 1)
while Running:
	Write(Address, Bank, 0xff)
	Data = Read(Address, Bank)
	Ret = Data & Byte
	if Ret == Byte:
		Cur = 1
	else:
		Cur = 0
	if Cur == 0 and Last == 1:
		Detect()
	Last = Cur

GPIO.output(Motor, 0)
GPIO.cleanup()
