#!/usr/bin/python

import smbus
class Adafruit_I2C :
  @staticmethod
  def getPiRevision():
    "Gets the version number of the Raspberry Pi board"
    try:
      with open('/proc/cpuinfo','r') as f:
        for line in f:
          if line.startswith('Revision'):
            return 1 if line.rstrip()[-1] in ['1','2'] else 2
    except:
      return 0
  @staticmethod
  def getPiI2CBusNumber():
    return 1 if Adafruit_I2C.getPiRevision() > 1 else 0
  def __init__(self, address, busnum=-1, debug=False):
    self.address = address
    print "Bus number = {}".format(busnum)
    print "Revision {}".format(self.getPiRevision())
    if Adafruit_I2C.getPiI2CBusNumber() == 0 and self.getPiRevision() == 1:
        busnum = 1;
    self.bus = smbus.SMBus( busnum if busnum >= 0 else Adafruit_I2C.getPiI2CBusNumber())
    self.debug = debug
  def reverseByteOrder(self, data):
    "Reverses the byte order of an int (16-bit) or long (32-bit) value"
    byteCount = len(hex(data)[2:].replace('L','')[::2])
    val       = 0
    for i in range(byteCount):
      val    = (val << 8) | (data & 0xff)
      data >>= 8
    return val
  def errMsg(self):
    return -1
  def write8(self, reg, value):
    try:
      self.bus.write_byte_data(self.address, reg, value)
    except IOError, err:
      return self.errMsg()
  def write16(self, reg, value):
    try:
      self.bus.write_word_data(self.address, reg, value)
    except IOError, err:
      return self.errMsg()
  def writeList(self, reg, list):
    try:
      self.bus.write_i2c_block_data(self.address, reg, list)
    except IOError, err:
      return self.errMsg()
  def readList(self, reg, length):
    "Read a list of bytes from the I2C device"
    try:
      results = self.bus.read_i2c_block_data(self.address, reg, length)
      return results
    except IOError, err:
      return self.errMsg()
  def readU8(self, reg):
    try:
      result = self.bus.read_byte_data(self.address, reg)
      return result
    except IOError, err:
      return self.errMsg()
  def readS8(self, reg):
    try:
      result = self.bus.read_byte_data(self.address, reg)
      if result > 127: result -= 256
      return result
    except IOError, err:
      return self.errMsg()

  def readU16(self, reg):
    try:
      hibyte = self.readU8(reg)
      lobyte = self.readU8(reg+1)
      result = (hibyte << 8) + lobyte
      return result
    except IOError, err:
      return self.errMsg()
  def readS16(self, reg):
    try:
      hibyte = self.readS8(reg)
      lobyte = self.readU8(reg+1)
      result = (hibyte << 8) + lobyte
      return result
    except IOError, err:
      return self.errMsg()
  def readU16Rev(self, reg):
    try:
      lobyte = self.readU8(reg)
      hibyte = self.readU8(reg+1)
      result = (hibyte << 8) + lobyte
      return result
    except IOError, err:
      return self.errMsg()
  def readS16Rev(self, reg):
    try:
      lobyte = self.readS8(reg)
      hibyte = self.readU8(reg+1)
      result = (hibyte << 8) + lobyte
      return result
    except IOError, err:
      return self.errMsg()
if __name__ == '__main__':
  try:
    bus = Adafruit_I2C(address=0)
  except:
    print ""
